<?php
/**
 * @package Smartsub (Smart Subscriptions) for Joomla 3
 * @version 1.0
 * @author Alex Polonski
 * @copyright (C)2014 - Alex Polonski
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 **/
defined('_JEXEC') or die;

abstract class PaymentPlugin extends JPlugin
{
	protected $autoloadLanguage = true;
	
	protected $payment_method;
	protected $api_credentials;
	protected $is_offline;
	
	// Smartsub component presense flag
	protected $componentInstalled = false;
	
	public function __construct(&$subject, $config)
	{
		parent::__construct($subject, $config);
		$this->api_credentials = $this->getApiCredentials();
		
		$this->componentInstalled = JComponentHelper::isEnabled('com_smartsub');
	}
	
	abstract protected function getApiCredentials();
	abstract public function onGetPaymentLinkData();
	abstract public function onInitPayment($paymentData = null);
	abstract public function onGetDetails($paymentData = null);
	abstract public function onDoTransaction($paymentData = null);
	abstract public function onLogTransaction($paymentData);
	
	public static function getAccountPaymentData($account_id, $user_id = null, $checkout_ts = null)
	{
		$db = JFactory:: getDbo();
		$db->setQuery('SELECT * FROM #__ss_accounts WHERE id = ' . (int)$account_id);
		try
		{
			$result = $db->loadObject();
			
		}
		catch(RuntimeException $e)
		{
			// log error
			return false;
		}
		
		$profile = JUserHelper::getProfile($user_id);
		if(!empty($profile->smartsub['account']))
		{
			$current_account_id = $profile->smartsub['account'];
			$valid_accounts = SmartsubHelper::getCurrentAccounts($user_id);
			$with_discounts = SmartsubHelper::getDiscounts(array($result), $valid_accounts, $current_account_id, $checkout_ts);
			$result = reset($with_discounts);
		}
		
		// Amount calculated with discounts must be rounded using precision,
		// set in component options, otherwise payment plugins may give errors
		// on "invalid amount"
		$params = JComponentHelper::getParams('com_smartsub', true);
		if($params)
		{
			$decimals = $params->get('currency_decimals', 2); // $$$ add this option to config... ?
			$result->amount = round($result->amount, $decimals);
		}
		
		return $result;
	}
	
	public function onGetPaymentMethodName()
	{
		JFactory::getLanguage()->load('plg_smartsub_' . $this->payment_method, JPATH_PLUGINS . '/smartsub/' . $this->payment_method);
		$result = new stdClass();
		$result->name = $this->payment_method;
		$result->title = JText::_('PLG_SMARTSUB_' . $result->name . '_TITLE');
		return $result;
	}
	
	// Plugins can overload this method - provide currency codes compatible with a payment method
	protected function getSupportedCurrencies()
	{
		return array();
	}
}
