<?php
/**
 * @package Smartsub (Smart Subscriptions) for Joomla 3
 * @version 1.1.9
 * @author Alex Polonski
 * @copyright (C)2014 - Alex Polonski
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 **/
defined('_JEXEC') or die;

define('COM_SMARTSUB_MAX_ACCOUNT_DESCRIPTIONS_VISIBLE', 5);
define('COM_SMARTSUB_DEMO_MODE', 0);

class SmartsubHelper extends JHelperContent
{
	public static function addSubmenu($vName = 'accounts')
	{
		JHtmlSidebar::addEntry(
			JText::_('COM_SMARTSUB_SUBMENU_ACCOUNTS'),
			'index.php?option=com_smartsub&view=accounts',
			$vName == 'accounts'
		);
		JHtmlSidebar::addEntry(
			JText::_('COM_SMARTSUB_SUBMENU_PAYMENTS'),
			'index.php?option=com_smartsub&view=payments',
			$vName == 'payments'
		);
	}
	
	public static function formatFloatNumber($val, $is_percentage = false)
	{
		if(is_numeric($val))
		{
			$params = JComponentHelper::getParams('com_smartsub');
			$decimal_point_char = JText::_('DECIMALS_SEPARATOR');
			$thousands_separator_char = JText::_('THOUSANDS_SEPARATOR');
			$decimals = $is_percentage ? 2 : $params->get('currency_decimals', 2);
			return number_format($val, $decimals, $decimal_point_char, $thousands_separator_char);
		}
		return '';
	}
	
	// Global list of supported currencies.
	public static function getCurrencies()
	{
		return array(
				'EUR' => '€',
				'USD' => '$',
				'GBP' => '£',
				'RUB' => '₽',
				'UAH' => '₴',
				'BYR' => 'р.',
				'CHF' => 'CHF',
				'JPY' => '¥',
				'ILS' => 'ILS',
				'CAD' => 'CAD',
				'AUD' => 'AUD',
				'HKD' => 'HKD',
				'NZD' => 'NZD',
				'SGD' => 'SGD',
				'SEK' => 'SEK',
				'PLN' => 'PLN'
				// add more...
		);
	}
	
	public static function formatPrice($val, $currencyID = null, $show_zero = false)
	{
		if(is_numeric($val))
		{
			if($show_zero || $val > 0)
			{
				$formatted = self::formatFloatNumber($val);
				$currencyID = empty($currencyID) ? JComponentHelper::getParams('com_smartsub')->get('default_currency', 'EUR') : $currencyID;
				
				// standard currency symbols
				$currencies = self::getCurrencies();
				
				if(isset($currencies[$currencyID]))
				{
					// we have a symbol
					$currency_char = $currencies[$currencyID];
				}
				else
				{
					// show currecy code as is
					$currency_char = $currencyID;
				}
				return $formatted . $currency_char;
			}
		}
		return '';
	}
	
	private static $expiration_data_cache;
	
	/**
	 * Get expiration data for an account, relative to $now_ts or "now"
	 * @param integer - $account_id
	 * @return array - expiration data (expires_ts, expiration_alert_time, now_ts)
	 */
	public static function getExpirationDataById($account_id = null, $now_ts = null)
	{
		if(empty($account_id))
		{
			return self::getExpirationData();
		}
		
		if(!isset(self::$expiration_data_cache[$account_id]))
		{
			$db = JFactory::getDbo();
			$db->setQuery('SELECT amount, valid_period, valid_period_unit FROM #__ss_accounts WHERE id = ' . (int)$account_id);
			try
			{
				$expiration_data = $db->loadObject();
				if($expiration_data->amount == 0)
				{
					$expiration_data->valid_period = 0;
				}
				self::$expiration_data_cache[$account_id] = self::getExpirationData($expiration_data->valid_period, $expiration_data->valid_period_unit, $now_ts);
			}
			catch(RuntimeException $e)
			{
				return self::getExpirationData();
			}
		}
		
		return self::$expiration_data_cache[$account_id];
	}
	
	// fast expiration data calculation - need to know $valid_period and $valid_period_unit
	public static function getExpirationData($valid_period = null, $valid_period_unit = null, $now_ts = null)
	{
		$now = JDate::getInstance($now_ts);
		// always return current timestamp in result, so that calling scripts can use it for
		// reliable date comparisons later in script
		$result = array('expires_ts' => 0, 'expiration_alert_time' => 0, 'valid_time' => 0, 'now_ts' => $now->getTimestamp());
		if(empty($valid_period))
		{
			return $result;
		}
		
		$params = JComponentHelper::getParams('com_smartsub');
		$expiration_alert_pct = $params->get('expiration_alert_pct', 10);
		$expiration_alert_min = $params->get('expiration_alert_min', 3600 * 24);
		$expiration_alert_max = $params->get('expiration_alert_max', 3600 * 24 * 14);
		
		$expires = clone($now);
		$expires->modify('+' . $valid_period . ' ' . $valid_period_unit);
		
		$result['expires_ts'] = $expires->getTimestamp();
		
		/* Account expiration time rounding is causing issues on account renewals
		 * RESERVED FOR FUTURE REFACTORING $$$
		$round_down = $valid_period_unit == 'hour' ? 300 : 3600;
		$result['expires_ts'] = intval($result['expires_ts'] / $round_down) * $round_down;
		*/
		
		$valid_time = $result['expires_ts'] - $now->getTimestamp();
		$expiration_alert_time = min($valid_time * $expiration_alert_pct / 100, $expiration_alert_max);
		$result['expiration_alert_time'] = max($expiration_alert_min, $expiration_alert_time);
		$result['valid_time'] = $valid_time;
		
		//$result['dbg_expires'] = JDate::getInstance($result['expires_ts'])->toSql();
		
		return $result;
	}
	
	public static function getCurrentAccounts($user_id)
	{
		if(empty($user_id))
		{
			return array();
		}
		$db = JFactory::getDbo();
		$now = JDate::getInstance()->toSql();
		// $$$ the same trick with overwriting PHP grouping - check if we can order by MAX(date) instead -
		// *** update: new policy ensures that there will be maximum 1 'paid' payment for the same account
		// and user, component settings doesn't matter
		$query = $db->getQuery(true);
		$query->select('DISTINCT(p.account_id), p.amount, p.currencyID, p.date, p.expires');
		$query->from('#__ss_payments AS p');
		$query->select('a.params');
		$query->leftJoin('#__ss_accounts AS a ON a.id = p.account_id');
		$query->where('p.status = ' . $db->Quote('paid'));
		$query->where('p.user_id = ' . (int)$user_id);
		$query->where('(p.expires = ' . $db->Quote($db->getNullDate()) . ' OR p.expires > ' . $db->Quote($now) . ')');
		$query->order('p.account_id ASC');
		$db->setQuery($query);
	
		try
		{
			$accounts = $db->loadObjectList('account_id');
		}
		catch(RuntimeException $e)
		{
			return array();
		}
		
		foreach($accounts as &$account)
		{
			$params = new JRegistry();
			$params->loadString($account->params);
			$account->params = $params;
		}
		
		return $accounts;
	}
	
	public static function getDiscounts($accounts, $valid_accounts, $current_account_id, $checkout_ts = null)
	{
		// set special prices for account switching (if set so in component configuration)
		$component_params = JComponentHelper::getParams('com_smartsub');
		$global_discount_unused_time = $component_params->get('discount_unused_time', 0);
		$global_discount_on_downgrade = $component_params->get('discount_on_downgrade', 0);
		$global_lock_expires_on_upgrade = $component_params->get('lock_expires_on_upgrade', 0);
		
		// if $checkout_ts is present in arguments, use it. Otherwise get current time
		$checkout_ts = !empty($checkout_ts) ? $checkout_ts : JDate::getInstance()->getTimestamp();
		
		// Only proceed with discount if there is at least 1 valid account and if current account is set for this user
		if(!empty($valid_accounts) && !empty($current_account_id))
		{
			// current account may have already expired (not an anomality, if account selection appears in admin user manager)
			$current_account = isset($valid_accounts[$current_account_id]) ? $valid_accounts[$current_account_id] : false;
			// only apply discounts if current account is valid
			if($current_account)
			{
				if(intval($current_account->expires) > 0)
				{
					// for accounts that expire we have to know unused time (time left before expiration)
					// then we apply discount basing on the part of amount proportional to unused time
					$expires_ts = JDate::getInstance($current_account->expires)->getTimestamp();
					$created_ts = JDate::getInstance($current_account->date)->getTimestamp();
						
					$total_account_time = $expires_ts - $created_ts;
					$unused_time = $expires_ts - $checkout_ts;
					$unused_part_k = $unused_time / $total_account_time;
				}
				else
				{
					// for accounts that never expire we consider the whole time as unused
					$unused_part_k = 1;
				}
		
				foreach($accounts as $n => $account)
				{
					// get params for the account (fallback to global params if configured so)
					$discount_unused_time = $current_account->params->get('discount_unused_time', -1);
					if($discount_unused_time == -1)
					{
						$discount_unused_time = $global_discount_unused_time;
					}
					$discount_on_downgrade = $current_account->params->get('discount_on_downgrade', -1);
					if($discount_on_downgrade == -1)
					{
						$discount_on_downgrade = $global_discount_on_downgrade;
					}
					$lock_expires_on_upgrade = $current_account->params->get('lock_expires_on_upgrade', -1);
					if($lock_expires_on_upgrade == -1)
					{
						$lock_expires_on_upgrade = $global_lock_expires_on_upgrade;
					}
					
					// calculate the discount
					$max_discount = $current_account->amount * $unused_part_k * $discount_unused_time / 100;
					
					// when applying discounts we will modify 'amount' property. We may need full
					// amount for later display, so save it here
					$accounts[$n]->full_amount = $accounts[$n]->amount;
					
					// Only act for paid account that is in current accounts array and with common currencyID,
					// so that we can calculate upgrade price
					if($account->amount > 0 && $account->id != $current_account_id && $account->currencyID == $current_account->currencyID)
					{
						// 'lock_expires_on_upgrade' feature implementation
						$account_expiration_data = self::getExpirationData($account->valid_period, $account->valid_period_unit, $checkout_ts);
						$valid_time = $account_expiration_data['valid_time'];
						if(!empty($valid_time) && $lock_expires_on_upgrade && !empty($unused_time))
						{
							// lock_expires_on_update is active and we have all conditions to apply it
							$amount = $account->amount * ($unused_time / $valid_time);
							// prepare new account expiration date equal to current account expiration date,
							// it will be applied if other upgrade conditions are met
							$expires_locked = $current_account->expires;
						}
						else
						{
							// use new account amount as is and do not alter expiration date
							$amount = $account->amount;
							$expires_locked = null;
						}
						
						// if 'discount_on_downgrade' is OFF, only upgrading (i.e. going to account
						// with a higher price) is qualified for discount
						if($amount >= $current_account->amount || $discount_on_downgrade)
						{
							// get amount with discount applied
							$diff = $amount - $max_discount;
							if($diff > 0)
							{
								// apply partial amount
								$accounts[$n]->discount_pct = $max_discount / $accounts[$n]->amount * 100;
								$accounts[$n]->discount = $max_discount;
								$accounts[$n]->amount = $diff;
							}
							else
							{
								// discount is greater than new account amount, so reset
								// amount and set discount as 100%
								// *** this is typical for downgrading, anyway if discount
								// on downgrade is not allowed we will never get here when
								// new account is cheaper than current one
								$accounts[$n]->discount_pct = 100.00;
								$accounts[$n]->discount = $accounts[$n]->amount;
								$accounts[$n]->amount = 0;
							}
							
							// apply expires correction (for lock_expires_on_update ON upgrade mode)
							$accounts[$n]->expires_locked = $expires_locked;
						}
						// in case of account upgrade/downgrade we remember old account_id
						$accounts[$n]->upgrade_from = $current_account_id;
					}
					elseif($current_account_id == $account->id)
					{
						// account allow_renew
						$discount_on_renew = $current_account->params->get('discount_on_renew', -1);
						if($discount_on_renew == -1)
						{
							$discount_on_renew = $component_params->get('discount_on_renew', 0);
						}
						$accounts[$n]->discount_pct = $discount_on_renew;
						$accounts[$n]->discount = $account->amount * $accounts[$n]->discount_pct / 100;
						$accounts[$n]->amount = $account->amount - $accounts[$n]->discount;
					}
					else
					{
						// no discount on incompatible accounts
						$accounts[$n]->discount = null;
						$accounts[$n]->discount_pct = null;
					}
					
					// Add a flag if this account is currently assigned to a user (may be used
					// to visually mark this account in accounts description list)
					// Anoter option: show expiration date for current account, but we still need to
					// show generic valid period, so that the user will see how much time he gets
					// after he renews (purchases) the same account.
					$accounts[$n]->is_current = $current_account_id && $account->id == $current_account_id;
				}
			}
		}
		
		return $accounts;
	}
	
	public static function findFreeAccount()
	{
		$db = JFactory::getDbo();
		$query = $db->getQuery(true);
	
		// we do not filter by published state here. So that a free account
		// can be assigned to a user even if not available for new registrations
		// When logged in user opens "edit profile", he will be
		// obliged to choose one of published accounts (paid) in order to be able
		// to save changes
		$query->select('a.id')
			->from('#__ss_accounts AS a')
			->innerJoin('#__usergroups AS b ON b.id = a.group_id')
			->where('a.amount = 0')
			->where('a.valid_period = 0')
			->order('a.ordering');
	
		$db->setQuery($query, 0, 1);
		$account_id = $db->loadResult();
	
		return !empty($account_id) ? $account_id : false;
	}
	
	public static function processEmailTpl($paymentData, $tpl, $changes = array())
	{
		$user = JUser::getInstance($paymentData->user_id);
		$app = JFactory::getApplication();
			
		$lang = JFactory::getLanguage();
		if($app->isAdmin())
		{	
			// if user has preferred language set in parameters, use it, otherwise we use current
			// backend user language
			$default_tag = $lang->getTag();
			// we can also use admin default language, so that it will not depend on the admin currently
			// logged in...
			//$default_tag = JComponentHelper::getParams('com_languages')->get('administrator');
			
			$lang_tag = $user->getParam('language', '') == '' ? $default_tag : $user->getParam('language');
		
			// load frontend component language
			$lang->load('com_smartsub', JPATH_SITE . '/components/com_smartsub', $lang_tag, true);
		}
		else 
		{
			// in frontend we rely on current display language for both translations
			// and template selection
			$lang_tag = $lang->getTag();
		}
			
		// get absolute images urls
		$component_params = JComponentHelper::getParams('com_smartsub');
		$logo_image_url = $header_image_url = $footer_image_url = '';
		
		$logo_image = $component_params->get('logo_image', '');
		$header_image = $component_params->get('header_image', '');
		$footer_image = $component_params->get('footer_image', '');
		if(!empty($logo_image))
		{
			$logo_image_url = JUri::root() . $logo_image;
		}
		if(!empty($header_image))
		{
			$header_image_url = JUri::root() . $header_image;
		}
		if(!empty($footer_image))
		{
			$footer_image_url = JUri::root() . $footer_image;
		}
			
		// load email template specific for current display language (fallback to en-GB)
		if(file_exists(JPATH_ADMINISTRATOR . '/components/com_smartsub/helpers/emailtpl/overrides/' . $lang_tag . '/' . $tpl . '.php'))
		{
			// look for override for current language and template
			$email_tpl = file_get_contents(JPATH_ADMINISTRATOR . '/components/com_smartsub/helpers/emailtpl/overrides/' . $lang_tag . '/' . $tpl . '.php');
		}
		elseif(file_exists(JPATH_ADMINISTRATOR . '/components/com_smartsub/helpers/emailtpl/' . $lang_tag . '/' . $tpl . '.php'))
		{
			// look for built-in template for current language
			$email_tpl = file_get_contents(JPATH_ADMINISTRATOR . '/components/com_smartsub/helpers/emailtpl/' . $lang_tag . '/' . $tpl . '.php');
		}
		else
		{
			// fallback to built-in en-GB template
			$email_tpl = file_get_contents(JPATH_ADMINISTRATOR . '/components/com_smartsub/helpers/emailtpl/en-GB/' . $tpl . '.php');
		}
		
		// Easy way to strip heading "<?php" block 
		$email_tpl = substr($email_tpl, strpos($email_tpl, '?>') + 2);
		
		// Cleanup images from template if no image of the kind is set in the component options
		// We rely on special-forms comment delimiters in format:
		// <!--XXXX_image_start ANY TEXT-->...<!--ANY TEXT XXXX_image_end-->
		// where XXXX stands for image role (header, footer or logo). All html between these
		// delimiters and the delimiters themselves will be removed from resulting template
		// *** to remove only image element, use regex
		//$email_tpl = preg_replace("/<img[^>]+{%XXXX_image_url}[^>]+\>/i", '', $email_tpl);
		if(empty($logo_image_url))
		{
			$email_tpl = preg_replace("#<!--logo_image_start(.*?)logo_image_end-->#is", '', $email_tpl);
		}
		if(empty($header_image_url))
		{
			$email_tpl = preg_replace("#<!--header_image_start(.*?)header_image_end-->#is", '', $email_tpl);
		}
		if(empty($footer_image_url))
		{
			$email_tpl = preg_replace("#<!--footer_image_start(.*?)footer_image_end-->#is", '', $email_tpl);
		}
		
		$payment_systems = self::getPaymentMethodNames();
		
		// *** new $changes param implementation preview
		$changes_notice = '';
		if(!empty($changes))
		{
			$changes_notice = '';
			foreach($changes as $key => $is_updated)
			{
				if($is_updated && !empty($paymentData->$key))
				{
					if($key == 'status')
					{
						$changes_notice .= '<strong>' . JText::_('COM_SMARTSUB_EMAIL_STATUS_NOTICE_' . $paymentData->$key) . '</strong>';
					}
					else
					{
						$changes_notice .= '<em>' . JText::_('COM_SMARTSUB_EMAIL_CHANGE_' . $key) . ': </em> ' . $paymentData->$key;
					}
					$changes_notice .= '<br />';
				}
			}
		}
		
		$status = !empty($paymentData->status) ? strtoupper($paymentData->status) : 'ERROR';
		
		$search = array(
				'{%username}',			// Joomla user login
				'{%useremail}',			// Joomla user email
				'{%transactionid}',		// Payment transaction ID
				'{%sitename}',			// Joomla site name
				'{%siteurl}',			// Joomla site URL
				'{%logo_image_url}',	// Logo image
				'{%header_image_url}',	// Header image
				'{%footer_image_url}',	// Footer image
				'{%price}',				// Price (with currency code)
				'{%account}',			// Subscription title
				'{%status}',			// Subscription status
				'{%expires}',			// Subscription expiration date
				'{%discount}',			// Absolute discount applied (with currency code)
				'{%discount_pct}',		// Discount in percentage
				'{%extra_info}',		// Extra info (admin's comment)
				'{%upgrade_from}',		// Previous account title (for subscription changes)
				'{%payment_method}',	// Payment method title
				'{%status_notice}',		// Generic status message
				'{%changes_notice}'		// Generic status updates message
		);

		// some $paymentData attributes may not be defined, e.g. when coming from "offline
		// transfer" bank data email generation. We can safely suppress warnings.
		$replace = array(
				$user->get('name'),
				$user->get('email'),
				$paymentData->transactionID,
				$app->get('sitename'),
				JUri::root(),
				$logo_image_url,
				$header_image_url,
				$footer_image_url,
				@$paymentData->price,
				@$paymentData->account,
				JText::_('COM_SMARTSUB_PAYMENT_STATUS_' . $status),
				@$paymentData->expires,
				@$paymentData->discount,
				@$paymentData->discount_pct,
				@$paymentData->extra_info,
				@$paymentData->old_account,
				isset($payment_systems[@$paymentData->system]) ? $payment_systems[$paymentData->system] : @$paymentData->system,
				'COM_SMARTSUB_EMAIL_STATUS_NOTICE_' . $status,
				$changes_notice
		);

		$body = str_replace($search, $replace, $email_tpl);
			
		// Normally email template is language-specific and will contain
		// all generic text already translated. We still leave an option
		// to have 'COM_SMARTSUB_EMAIL_<YOUR_KEY_SUFFIX>' strings in the
		// text. If found they will be replaced with translations from
		// language file
		$matches = array();
		preg_match_all('#COM_SMARTSUB_EMAIL_[A-Z0-9_]+#', $body, $matches);
			
		if(!empty($matches[0]))
		{
			$search = array();
			$replace = array();
			foreach($matches[0] as $lang_key)
			{
				$search[] = $lang_key;
				$replace[] = JText::_($lang_key);
			}
			$body = str_replace($search, $replace, $body);
		}
		
		return $body;
	}
	
	public static function paymentNotification($data, $isNew = true, $changes = array())
	{
		// send notification to user
		
		$app = JFactory::getApplication();
		$db = JFactory::getDbo();
		
		$task = $app->input->get('task');
		$component = $app->input->getWord('option', 'com_users');
		
		// get new account data
		JTable::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_smartsub/tables');
		$account_table = JTable::getInstance('Account', 'SmartsubTable');
		if(empty($data['account_id']) || !$account_table->load($data['account_id']))
		{
			// log error here...
			return false;
		}
		
		// populate $data with more properties for display
		$data['account'] = $account_table->title;
		
		// Prices
		// actual amount paid (with discounts applied)
		$data['price'] = self::formatPrice($data['amount'], $data['currencyID'], true);
		// full account price
		$data['full_price'] = self::formatPrice($account_table->amount, $data['currencyID']);
		
		// Discounts
		$show_discount = $data['discount'] > 0 && $account_table->amount > 0;
		// discount in percents
		$data['discount_pct'] = $show_discount ? SmartsubHelper::formatFloatNumber($data['discount'] / $account_table->amount * 100, true) . '%' : '';
		// absolute discount
		$data['discount'] = $show_discount ? self::formatPrice($data['discount'], $data['currencyID']) : '';
		
		// Old account title (if any)
		if(!empty($data['upgrade_from']) && $account_table->load($data['upgrade_from']))
		{
			$data['old_account'] = $account_table->title;
		}
		else 
		{
			$data['old_account'] = '';
		}
		
		/* *** new feature: we have $data['task'] at this point
		 * 'register' - new registration
		 * 'profile' - profile edit
		 * 'preaccess' - free account assigned for offline payments
		 * 'reset' - reset to free account in case of expired or paused subscription
		 * 
		 */
		
		// Payments with 'administrative' payment method are always created by
		// admin, so force 'admin' template
		if($data['system'] == 'administrative')
		{
			$tpl = 'admin';
		}
		else // all other cases
		{
			$task_tpl_map = array(
					// at the moment we use the same template for "expired" and "block" events
					'expired' => 'reset',
					'block' => 'reset',
					'preaccess' => 'preaccess',
					'register' => 'signup',
					'profile' => $isNew ? 'upgrade' : 'status'
			);
			$tpl = $task_tpl_map[$data['task']];
		}
		
		// get user's timezone, if not set, use system timezone
		$user = JUser::getInstance($data['user_id']);
		$timezone = $user->getParam('timezone', $app->get('offset'));
		
		$lang = JFactory::getLanguage();
		$lang_tag = $user->getParam('language', '') == '' ? $lang->getTag() : $user->getParam('language');
		
		// load frontend component language
		$lang->load('com_smartsub', JPATH_SITE . '/components/com_smartsub', $lang_tag);
		
		$data['expires'] = empty($data['expires']) || $data['expires'] == $db->getNullDate() ?
			JText::_('COM_SMARTSUB_ACCOUNT_EXPIRES_NEVER') :
			JHtml::date($data['expires'], JText::_('COM_SMARTSUB_DATE_TIME_FORMAT'), $timezone);
		
		$body = self::processEmailTpl((object)$data, $tpl, $changes);
		
		$subject = ($isNew ?
				JText::_('COM_SMARTSUB_PAYMENT_NOTIFICATION_SUBJECT_NEW') :
				JText::_('COM_SMARTSUB_PAYMENT_NOTIFICATION_SUBJECT_EDIT')
		) . (!empty($data['transactionID']) ? ' - ' .$data['transactionID'] : '');
			
		// send notification
		self::sendNotification($data['user_id'], $subject, $body);
	}
	
	public static function sendNotification($user_id, $subject, $body, $toAdmin = false)
	{
		$mail = JFactory::getMailer();
			
		$user = JUser::getInstance($user_id);
		$app = JFactory::getApplication();
		
		// Always send email from site email (so that SMTP server doesn't report error
		// on "email not owned")
		$mailfrom = $app->get('mailfrom');
		$fromname = $app->get('fromname');
		$mail->setSender(array($mailfrom, $fromname));
		if($toAdmin)
		{
			// notification to admin
			$mail->addRecipient(JStringPunycode::emailToPunycode($app->get('mailfrom')));
		}
		else 
		{
			// notification to user
			$mail->addRecipient(JStringPunycode::emailToPunycode($user->get('email')));
		}
		$sitename = $app->get('sitename');

		$mail->setSubject($sitename . ': ' . $subject);
		$mail->setBody(stripslashes($body));
		$mail->Encoding = 'bas' . 'e64'; // JED Checker workaround
		$mail->isHtml(true);
			
		$sent = $mail->Send();
		return $sent;
	}
	
	protected static $payment_systems = null;
	public static function getPaymentMethodNames()
	{
		if(!isset(self::$payment_systems))
		{
			// get payment type titles
			$dispatcher	= JDispatcher::getInstance();
			JPluginHelper::importPlugin('smartsub');
			$result = $dispatcher->trigger('onGetPaymentMethodName');
			self::$payment_systems = array();
			foreach($result as $system)
			{
				self::$payment_systems[$system->name] = $system->title;
			}
			self::$payment_systems['administrative'] = JText::_('COM_SMARTSUB_PAYMENT_ADMINISTRATIVE');
			self::$payment_systems['gratis'] = JText::_('COM_SMARTSUB_PAYMENT_GRATIS');
			self::$payment_systems['none'] = JText::_('COM_SMARTSUB_PAYMENT_NONE');
		}
		return self::$payment_systems;
	}
	
	public static function getInternalTransactionID($username, $account_id, $prefix = 'ADMIN')
	{
		return $prefix . JDate::getInstance()->format('ymd') . '-' . strtoupper($username) . str_pad($account_id, 3, '0', STR_PAD_LEFT);
	}
}
