<?php
/**
 * @package Smartsub (Smart Subscriptions) for Joomla 3
 * @version 1.0
 * @author Alex Polonski
 * @copyright (C)2014 - Alex Polonski
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 **/
defined('_JEXEC') or die;

require_once JPATH_ADMINISTRATOR . '/components/com_smartsub/helpers/smartsub.php';

class SmartsubModelAccount extends JModelAdmin
{
	protected $text_prefix = 'COM_SMARTSUB_ACCOUNT';

	protected function canDelete($record)
	{
		if (!empty($record->id))
		{
			if ($record->published != -2)
			{
				return;
			}
			return parent::canDelete($record);
		}
	}

	public function getTable($type = 'Account', $prefix = 'SmartsubTable', $config = array())
	{
		return JTable::getInstance($type, $prefix, $config);
	}

	public function getForm($data = array(), $loadData = true)
	{
		// Get the form.
		$form = $this->loadForm('com_smartsub.account', 'account', array('control' => 'jform', 'load_data' => $loadData));

		if (empty($form))
		{
			return false;
		}

		// Modify the form based on access controls.
		if (!$this->canEditState((object) $data))
		{
			// Disable fields for display.
			$form->setFieldAttribute('ordering', 'disabled', 'true');
			$form->setFieldAttribute('published', 'disabled', 'true');
			
			// Disable fields while saving.
			// The controller has already verified this is a record you can edit.
			$form->setFieldAttribute('ordering', 'filter', 'unset');
			$form->setFieldAttribute('published', 'filter', 'unset');
		}
		
		return $form;
	}

	protected function loadFormData()
	{
		// Look at post for form data (for onchange="this.form.submit();")
		$data = JRequest::getVar('jform', array(), 'post', 'array');
		if(empty($data))
		{
			// Check the session for previously entered form data.
			$data = JFactory::getApplication()->getUserState('com_smartsub.edit.account.data', array());
		}
		if (empty($data))
		{
			$data = $this->getItem();
		}

		$this->preprocessData('com_smartsub.account', $data);

		return $data;
	}

	public function getItem($pk = null)
	{
		$item = parent::getItem($pk);
		

		return $item;
	}

	protected function prepareTable($table)
	{
		$table->title = htmlspecialchars_decode($table->title, ENT_QUOTES);
		$table->alias = JApplication::stringURLSafe($table->alias);

		if (empty($table->alias))
		{
			$table->alias = JApplication::stringURLSafe($table->title);
		}

		if (empty($table->id))
		{
			// Set ordering to the last item if not set
			if (empty($table->ordering))
			{
				$db = JFactory::getDbo();
				$query = $db->getQuery(true)
					->select('MAX(ordering)')
					->from($db->quoteName('#__ss_accounts'));

				$db->setQuery($query);
				$max = $db->loadResult();

				$table->ordering = $max + 1;
			}
		}
		if(empty($table->amount))
		{
			// no expiration from free accounts
			$table->valid_period = 0;
		}
	}

	public function save($data)
	{
		$app = JFactory::getApplication();

		// Alter the title for save as copy
		if ($app->input->get('task') == 'save2copy')
		{
			list($name, $alias) = $this->generateNewTitle(null, $data['alias'], $data['title']);
			$data['title']	= $name;
			$data['alias']	= $alias;
			$data['published']	= 0;
		}

		// validate data
		if($data['amount'] < 0)
		{
			$this->setError(JText::_('COM_SMARTSUB_ACCOUNT_ERROR_PRICE'));
			return false;
		}
		
		$result = parent::save($data);
		
		$id = $this->getState('account.id');
		if($data['is_default'])
		{
			try 
			{
				$this->setDefault($id);
			}
			catch(RuntimeException $e)
			{
				$this->setError($e->getMessage());
				return false;
			}
		}

		return $result;
	}
	
	public function setDefault($id = 0)
	{
		$user = JFactory::getUser();
		$db   = $this->getDbo();

		// Access checks.
		if (!$user->authorise('core.edit.state', 'com_smartsub'))
		{
			throw new Exception(JText::_('JLIB_APPLICATION_ERROR_EDITSTATE_NOT_PERMITTED'));
		}

		$account = JTable::getInstance('Account', 'SmartsubTable');

		if (!$account->load((int) $id))
		{
			throw new Exception(JText::_('COM_SMARTSUB_ERROR_ACCOUNT_NOT_FOUND'));
		}

		$db->setQuery(
			'UPDATE #__ss_accounts' .
			' SET is_default = 0'
		);
		$db->execute();

		// Set the new home style.
		$db->setQuery(
			'UPDATE #__ss_accounts' .
			' SET is_default = 1' .
			' WHERE id = ' . (int) $id
		);
		$db->execute();

		// Clean the cache.
		$this->cleanCache();

		return true;
	}
	
	protected function generateNewTitle($category_id /* is ignored */, $alias, $name)
	{
		// Alter the title & alias
		$table = $this->getTable();

		while ($table->load(array('alias' => $alias)))
		{
			if ($name == $table->title)
			{
				$name = JString::increment($name);
			}

			$alias = JString::increment($alias, 'dash');
		}

		return array($name, $alias);
	}
}
