<?php
/**
 * @package Smartsub (Smart Subscriptions) for Joomla 3
 * @version 1.0
 * @author Alex Polonski
 * @copyright (C)2014 - Alex Polonski
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 **/
defined('JPATH_BASE') or die;

require_once JPATH_ADMINISTRATOR . '/components/com_smartsub/helpers/smartsub.php';

JFormHelper::loadFieldClass('list');

class JFormFieldCurrency extends JFormFieldList
{
	public $type = 'Currency';

	protected function getOptions()
	{
		// hardcoded currency list from helper
		$currencies = SmartsubHelper::getCurrencies();
		
		// for component configuration - all currencies available
		$config_mode = ((string)$this->element['config_mode'] == '1' || (string)$this->element['config_mode'] == 'true') ? 1 : 0;
		
		$params = JComponentHelper::getParams('com_smartsub');
		$allowed_currencies = $config_mode ? array() : $params->get('allowed_currencies', array());
		
		$options = array();
		
		foreach($currencies as $code => $char)
		{
			if(!empty($allowed_currencies) && !in_array($code, $allowed_currencies))
			{
				continue;
			}
			$option = new stdClass();
			$option->value = $code;
			
			$option->text = $char . ' (' . $code . ')';
			$options[] = $option;
		}
		
		// Merge any additional options in the XML definition.
		$options = array_merge(parent::getOptions(), $options);

		return $options;
	}
	
	/*
	 * Workaround for missing online help for Smart Subscriptions component
	 * Although we can easily control "Help" button display and behavior in all
	 * backend views in the component, we have no power to change "Help" button
	 * that appears in Smart Subscriptions configuration (options) view.
	 * This button is generated automatically by com_config core component and
	 * will try to reference joomla online help server (configured in helpsites.xml),
	 * thus leading to a page that will never exist.
	 * 
	 * The solution is to hide "help" button in this case. We rely on this custom
	 * field, as it will always be rendered on Smart Subscriptions comfiguration
	 * screen and the script below will be executed.
	 * 
	 * Later we can render our own "help" button, using JToolbar::help('some_link', true)
	 * to force local help files to be loaded.
	 */
	protected function getInput()
	{
		$html = parent::getInput();
		
		if(JFactory::getApplication()->input->getCmd('option', '') == 'com_config')
		{
			JFactory::getDocument()->addScriptDeclaration('
				jQuery(document).ready(function($) {
					$("button[rel=\'help\']").hide();
					
				});
			');
		}
		return $html;
	}
}
