<?php
/**
 * @package Smartsub (Smart Subscriptions) for Joomla 3
 * @version 1.0
 * @author Alex Polonski
 * @copyright (C)2014 - Alex Polonski
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 **/
defined('_JEXEC') or die;

require_once(JPATH_ADMINISTRATOR . '/components/com_smartsub/helpers/smartsub.php');

class SmartsubTablePayment extends JTable
{
	public function __construct(&$db)
	{
		parent::__construct('#__ss_payments', 'id', $db);
	}

	public function store($updateNulls = false)
	{
		$db = $this->getDbo();
		if(!empty($this->id))
		{
			// we have existing row
			$db->setQuery('SELECT * FROM #__ss_payments WHERE id = ' . (int)$this->id);
			try
			{
				$existing = $db->loadObject();
				if(empty($existing->id))
				{
					throw new Exception(JText::_('JLIB_DATABASE_ERROR_EMPTY_ROW_RETURNED'));
				}
			}
			catch(Exception $e)
			{
				$this->setError($e->getMessage());
				return false;
			}
			
			$isNew = false;
			$changes = array(
					'account'	=> $this->account_id != $existing->account_id,
					'status'	=> $this->status != $existing->status,
					'expires'	=> $this->expires != $existing->expires,
					'price'		=> $this->amount != $existing->amount || $this->currencyID != $existing->currencyID,
					'extra_info'=> $this->extra_info != $existing->extra_info
 			);
		}
		else
		{
			$isNew = true;
			$changes = array();
		}
		
		/* Reserved - store modification date $$$
		if($this->id)
		{
			$this->modified = JDate::getInstance()->toSql();
		}
		*/
		
		if(parent::store($updateNulls))
		{
			// After we save an order we have to sanitize payments table:
			// if current payment status is 'paid', we may need to set all other
			// 'paid' payments for the same user to a special 'archived' status
			$params = JComponentHelper::getParams('com_smartsub');
			$easy_accounts_switch = $params->get('easy_accounts_switch', 0);
			if($this->status == 'paid')
			{
				$query = $db->getQuery(true);
				$query->update($db->QuoteName($this->getTableName()));
				$query->set('status = ' . $db->Quote('archived'));
				
				// update all paid rows and those pending that refer to the same account
				$query->where('(status = ' . $db->Quote('paid') . ' OR (status = ' . $db->Quote('pending') . ' AND account_id = ' . (int)$this->account_id . '))');
				
				// only for this user
				$query->where('user_id = ' . (int)$this->user_id);
				// exclude current record
				$query->where('id <> ' . (int)$this->id);
				
				if($easy_accounts_switch == 1)
				{
					// if easy account switch is ON only update rows for the same account, so
					// that other paid subscriptions are not affected
					$query->where('account_id = ' . (int)$this->account_id);
				}
				$db->setQuery($query);
	
				try 
				{
					$db->execute();
				}
				catch(RuntimeException $e)
				{
					$this->setError($e->getMessage());
					return false;
				}
			}
			
			// we get to this point whenever a payment is added/modified in to/in
			// #__ss_payments table. This is a perfect place to send payment
			// notifications to admins/users

			// Only send notification for new or modified payments, except those with
			// archived status
			if(($isNew || in_array(true, $changes)) && $this->status != 'archived')
			{
				SmartsubHelper::paymentNotification($this->getProperties(true), $isNew, $changes);
				
				// Also send notification to admin on all offline payments in initial
				// (pending) status
				if($this->offline && $this->status == 'pending')
				{
					$this->adminPendingAlert();
				}
			}
			
			return true;
		}
		return false;
	}

	public function check()
	{

		return true;
	}
	
	private function adminPendingAlert()
	{
		$user = JUser::getInstance($this->user_id);
		
		// We have to use site default language to generate admin alerts
		$default_lang = JComponentHelper::getParams('com_languages')->get('administrator');
		JFactory::getLanguage()->load('com_smartsub', JPATH_ADMINISTRATOR . '/components/com_smartsub', $default_lang, true);

		$subject = JText::sprintf('COM_SMARTSUB_PAYMENT_PENDING_ADMIN_EMAIL_SUBJECT', $this->transactionID);
		$body = JText::sprintf('COM_SMARTSUB_PAYMENT_PENDING_ADMIN_EMAIL_BODY', $this->transactionID, $this->id, $user->get('username'), JHtml::date($this->date, 'Y-m-d H:i:s'), JText::_('COM_SMARTSUB_PAYMENT_STATUS_' . $this->status));
					$sent = SmartsubHelper::sendNotification($this->user_id, $subject, $body, true);
	}
}
