<?php
/**
 * @package Smartsub (Smart Subscriptions) for Joomla 3
 * @version 1.1.6
 * @author Alex Polonski
 * @copyright (C)2014 - Alex Polonski
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 **/
defined('_JEXEC') or die;

require_once(JPATH_ADMINISTRATOR . '/components/com_smartsub/helpers/smartsub.php');

define('SMARTSUB_ORDER_GRACE_TIME', 1200);

class SmartsubControllerPayment extends JControllerLegacy
{
	// New account ID selected in user profile
	protected $new_account;
	
	public function __construct($config)
	{
		parent::__construct($config);
		
		$this->registerTask('register', 'apply');
		$this->registerTask('profile', 'apply');
	}
	
	// this method will be called for both register and profile ***
	public function apply()
	{
		/*
		 * standard tasks:
		 * 		profile.save - for profile modification
		 * 		registration.register - for new registrations
		 */
		
		// Check for request forgeries.
		JSession::checkToken() or jexit(JText::_('JINVALID_TOKEN'));
		
		$task = $this->getTask();
		
		$user = JFactory::getUser();
		if($task == 'register' && $user->get('guest') != 1)
		{
			$this->setRedirect(JUri::base());
			return;
		}
		
		if($task == 'profile' && $user->get('guest') == 1)
		{
			$this->setRedirect(JUri::base());
			return;
		}
		
		$userId	= (int) $user->get('id');
		JFactory::getLanguage()->load('com_users');

		$app	= JFactory::getApplication();
		
		// Get the user data.
		$requestData = $this->input->post->get('jform', array(), 'array');
		
		JModelLegacy::addIncludePath(JPATH_ROOT . '/components/com_users/models', 'UsersModel');
		if($task == 'register')
		{
			$model	= JModelLegacy::getInstance('Registration', 'UsersModel', array());
			
			// for new users: create the user with new account
			$this->new_account = $requestData['smartsub']['account'];
		}
		else
		{
			$model	= JModelLegacy::getInstance('Profile', 'UsersModel', array());
			// Force the ID to this user.
			$requestData['id'] = $userId;
			
			// for existing users: for now save the updated profile with old account
			// (if any). We store the new account here to be able to get payment data
			// for the account requested and start payment. On successfull payment
			// we will update the user with the new account id in profile
			$this->new_account = $requestData['smartsub']['account'];
			$profile = JUserHelper::getProfile($userId);
			if(!empty($profile->smartsub['account']))
			{
				// temporarly revert account to old one
				$requestData['smartsub']['account'] = $profile->smartsub['account'];
			}
		}

		// Validate the posted data.
		JForm::addFormPath(JPATH_ROOT . '/components/com_users/models/forms');
		$form	= $model->getForm();

		if(!$form)
		{
			JError::raiseError(500, $model->getError());
			return false;
		}
		
		$data	= $model->validate($form, $requestData);

		// get checkout time from input, so that amount (with discount) is consistent with
		// the value displayed in edit profile form
		$checkout_ts = $this->input->get('pr-checkout-ts', '');
		
		// For profile edit task checkout_ts cannot be too far in past. If so, abort with message
		if($task == 'profile' && $checkout_ts && JDate::getInstance()->getTimestamp() - $checkout_ts > SMARTSUB_ORDER_GRACE_TIME)
		{
			$model->setError(JText::_('COM_SMARTSUB_PAYMENT_ERROR_GRACE_TIME'));
			$data = false;
		}
		
		// Check for validation errors.
		if ($data === false)
		{
			// Get the validation messages.
			$errors	= $model->getErrors();

			// Push up to three validation messages out to the user.
			for ($i = 0, $n = count($errors); $i < $n && $i < 3; $i++)
			{
				if ($errors[$i] instanceof Exception)
				{
					$app->enqueueMessage($errors[$i]->getMessage(), 'warning');
				}
				else
				{
					$app->enqueueMessage($errors[$i], 'warning');
				}
			}

			// Save the data in the session.
			if($task == 'register')
			{
				$app->setUserState('com_users.registration.data', $requestData);
				// Redirect back to the registration screen.
				$this->setRedirect(JRoute::_('index.php?option=com_users&view=registration', false));
			}
			else
			{
				$app->setUserState('com_users.edit.profile.data', $requestData);
				$userId = (int) $app->getUserState('com_users.edit.profile.id');
				// Redirect back to the registration screen.
				$this->setRedirect(JRoute::_('index.php?option=com_users&view=profile&layout=edit&user_id=' . $userId, false));
			}

			return false;
		}

		// Attempt to save the data.
		if($task == 'register')
		{
			$return	= $model->register($data);
		}
		else
		{
			$return	= $model->save($data);
		}

		// Check for errors.
		if($return === false)
		{
			// Save the data in the session.
			if($task == 'register')
			{
				$app->setUserState('com_users.registration.data', $data);
				// Redirect back to the edit screen.
				$this->setMessage(JText::sprintf('COM_USERS_REGISTRATION_SAVE_FAILED', $model->getError()), 'warning');
				$this->setRedirect(JRoute::_('index.php?option=com_users&view=registration', false));
			}
			else
			{
				$app->setUserState('com_users.edit.profile.data', $data);
				$userId = (int) $app->getUserState('com_users.edit.profile.id');
				// Redirect back to the edit screen.
				$this->setMessage(JText::sprintf('COM_USERS_PROFILE_SAVE_FAILED', $model->getError()), 'warning');
				$this->setRedirect(JRoute::_('index.php?option=com_users&view=profile&layout=edit&user_id=' . $userId, false));
			}
			return false;
		}

		// Always save validated data to session. We will clean up after successfull payment
		// or if the payment is cancelled
		if($task == 'register')
		{
			$app->setUserState('com_users.registration.data', $data);
		}
		else
		{
			$app->setUserState('com_users.edit.profile.data', $data);
		}
		
		// The user is created successfully, let's proceed with payment
		
		// get the created user object
		$new_user = JUser::getInstance($data['username']);
		
		$paymentData = PaymentPlugin::getAccountPaymentData($this->new_account, $new_user->get('id'), $checkout_ts);
		if(!$paymentData)
		{
			// error here - account not valid
			return false;
		}
		
		$paymentMethod = $app->input->getCmd('payment_method', '');
		$paymentData->payment_method = $paymentMethod;
		
		// inject user_id into the payment data, so that we can delete user in case
		// of payment cancellation or error
		$paymentData->user_id = $new_user->get('id');
		$paymentData->username = $new_user->get('username');
		
		// inject current task (register/profile) into payment data. We can later
		// use this property to redirect user correctly in case of payment cancellation
		// or error 
		$paymentData->task = $task;
		
		if($paymentData->amount == 0)
		{
			$paymentData->status = 'paid';
			$paymentData->complete = true;
		}
		
		// save updated payment data
		$app->setUserState('com_smartsub.payment_data', $paymentData);
		
		if($paymentData->amount > 0)
		{
			// start with transaction
			$this->setRedirect(JRoute::_('index.php?option=com_smartsub&task=payment.init&payment_method=' . $paymentMethod, false));
		}
		else // free upgrade
		{
			// start with transaction
			$this->setRedirect(JRoute::_('index.php?option=com_smartsub&task=payment.dotransaction', false));
		}
		// nothing more to do here
		return true;
	}
	
	public function init()
	{
		$app = JFactory::getApplication();
		$paymentData = $app->getUserState('com_smartsub.payment_data');
		
		if(empty($paymentData))
		{
			// we may get here if "back" button is pressed in browser while displaying
			// payment gateway page open in another context (when current session data
			// is not available). Can we handle this class of scenarios better?
			$app->enqueueMessage(JText::_('COM_SMARTSUB_PAYMENT_CANCELLED'));
			return false;
		}
		
		$payment_method = $paymentData->payment_method;
		
		$dispatcher = JDispatcher::getInstance();
		if(!JPluginHelper::importPlugin('smartsub', $payment_method))
		{
			// plugin not found
			self::cleanUpUser($paymentData);
			return false;
		}
		
		$result = $dispatcher->trigger('onInitPayment', array($paymentData));
		if (empty($result) || in_array(false, $result, true))
		{
			// Plugin will have to raise it's own error or throw an exception.
			self::cleanUpUser($paymentData);
			$this->setRedirect(JUri::base());
			return false;
		}
		
		$result = reset($result);
		$paymentData = $result['payment_data'];
		$redirectURL = $result['redirect_url'];
		
		$app->setUserState('com_smartsub.payment_data', $paymentData);
		
		// Redirect
		if(!empty($redirectURL))
		{
			JFactory::getApplication()->redirect($redirectURL);
		}
		return true;
	}
	
	public function cancel()
	{
		$app = JFactory::getApplication();
		$paymentData = $app->getUserState('com_smartsub.payment_data');
		
		$this->setMessage(JText::_('COM_SMARTSUB_PAYMENT_CANCELLED'), 'info');
		self::cleanUpUser($paymentData);
		
		if($paymentData->task == 'register')
		{
			$this->setRedirect(JRoute::_('index.php?option=com_users&view=registration', false));
		}
		else
		{
			$userId = $paymentData->user_id;
			$this->setRedirect(JRoute::_('index.php?option=com_users&view=profile&layout=edit&user_id=' . $userId, false));
		}
		
		// flush payment data from session on cancel - CHECK THIS!!!!!
		$app->setUserState('com_smartsub.payment_data', null);
		
		return true;
	}
	
	public function getdetails()
	{
		$app = JFactory::getApplication();
		$paymentData = $app->getUserState('com_smartsub.payment_data');
		
		if(empty($paymentData))
		{
			$paymentData = self::getPaymentDataFromRequest();
		}
		
		$payment_method = $paymentData->payment_method;
		
		$dispatcher = JDispatcher::getInstance();
		if(!JPluginHelper::importPlugin('smartsub', $payment_method))
		{
			// plugin not found
			self::cleanUpUser($paymentData);
			return false;
		}
		
		$result = $dispatcher->trigger('onGetDetails', array($paymentData));
		if (empty($result) || in_array(false, $result, true))
		{
			// Plugin will have to raise it's own error or throw an exception.
			self::cleanUpUser($paymentData);
			$this->setRedirect(JUri::base());
			return false;
		}
		
		$result = reset($result);
		$paymentData = $result['payment_data'];
		$redirectURL = $result['redirect_url'];
		
		$app->setUserState('com_smartsub.payment_data', $paymentData);
		
		// Redirect
		if(!empty($redirectURL))
		{
			JFactory::getApplication()->redirect($redirectURL);
		}
		
		if(!empty($result['close_app']))
		{
			ob_clean();
			echo $result['close_app'];
			$app->close();
		}
		
		return true;
	}
	
	public function dotransaction()
	{
		$app = JFactory::getApplication();
		$paymentData = $app->getUserState('com_smartsub.payment_data');
		$payment_method = $paymentData->payment_method;
		
		if($paymentData->amount > 0)
		{
			// only trigger payment plugin if amount is grater than zero
			$dispatcher = JDispatcher::getInstance();
			if(!JPluginHelper::importPlugin('smartsub', $payment_method))
			{
				// plugin not found
				self::cleanUpUser($paymentData);
				return false;
			}
			
			$result = $dispatcher->trigger('onDoTransaction', array($paymentData));
			if (empty($result) || in_array(false, $result, true))
			{
				// Plugin will have to raise it's own error or throw an exception.
				self::cleanUpUser($paymentData);
				$this->setRedirect(JUri::base());
				return false;
			}
			
			$result = reset($result);
			$paymentData = $result['payment_data'];
		}
		
		$user = JUser::getInstance($paymentData->user_id);
		$userId = $paymentData->user_id;
		
		// we have to implement 'redirect_url' support: standard (for the same session calls) -
		//		should work as it is done now, but hidden requests (bypassing the browser, e.g.
		//		WebMoney) should not redirect at all. Looks that this extra redirection is not a 
		//		problem, but it makes network activity dirty...

		$complete = !empty($paymentData->complete) ? true : false;
		
		if($complete)
		{
			// for complete payments we always log transaction (also for offline payments)
			$payment_id = self::logTransaction($paymentData);
			if(!$payment_id)
			{
				// contact administrator! Error message already added in logTransaction()
				$this->setRedirect(JUri::root());
				return false;
			}
			
			// payment success
			$task = $paymentData->task;
			
			if($task == 'register')
			{
				if($paymentData->status == 'paid')
				{
					// activate account
					$user->set('activation', '');
					$user->set('block', '0');
					
					$result = $user->save(true);
					if(!$result)
					{
						// contact administrator! payment done but profile was not updated
						$this->setMessage(JText::_('COM_SMARTSUB_PAYMENT_ACCOUNT_ACTIVATION_FAILED'), 'warning');
					}
					else 
					{
						$this->setMessage(JText::_('COM_SMARTSUB_PAYMENT_TRANSACTION_SUCCESS_LOGIN'), 'info');
					}
					$this->setRedirect(JRoute::_('index.php?option=com_users&view=login', false));
				}
				else 
				{
					// pending state
					// For new registrations we have to assign a free account to the new user, so
					// that he can login right now. When the payment is received administrator will
					// mark it as 'paid' and the account purchased will be assigned to the user
					$free_account = SmartsubHelper::findFreeAccount();
						
					// try to change user account to a free one
					if($free_account)
					{
						// revert to free account in profile
						$profile = JUserHelper::getProfile($userId);
						$profile->smartsub['account'] = $free_account;
						
						// set special task flag (will not be saved in real profile)
						$profile->smartsub['account_reset'] = 'preaccess';
						
						$user->set('smartsub', $profile->smartsub);
						$user->save(true);
						$this->setMessage(JText::_('COM_SMARTSUB_PAYMENT_ORDER_SUCCESS'), 'info');
						$this->setRedirect(JRoute::_('index.php?option=com_users&view=login', false));
					}
					else
					{
						// no free account - user will not be able to login until administrator
						// marks the payment as 'paid'
						$this->setMessage(JText::_('COM_SMARTSUB_PAYMENT_ORDER_SUCCESS_NOLOGIN'), 'info');
						$this->setRedirect(JUri::root());
					}
				}
				
				// Flush the data from the session.
				$app->setUserState('com_users.registration.data', null);
			}
			else
			{ // 'edit profile' task
				if($paymentData->status == 'paid')
				{
					// this is online payment success for user account update
					JModelLegacy::addIncludePath(JPATH_COMPONENT_ADMINISTRATOR . '/models');
					$model = $this->getModel('Payment');
					
					// Call payment model setPaid(). User profile will be updated in model
					try 
					{
						$model->setPaid($payment_id, 'paid');
						$this->setMessage(JText::_('COM_SMARTSUB_PAYMENT_TRANSACTION_SUCCESS'), 'info');
					}
					catch(Exception $e)
					{
						// contact administrator! payment done but payment row not updated
						$this->setMessage(JText::_('COM_SMARTSUB_PAYMENT_STATUS_CHANGE_FAILED'), 'warning');
					}			
				}
				else 
				{
					// pending state
					$this->setMessage(JText::_('COM_SMARTSUB_PAYMENT_ORDER_SUCCESS'), 'info');
				}
				
				// Clear the profile data from the session.
				$app->setUserState('com_users.edit.profile.id', null);
				$app->setUserState('com_users.edit.profile.data', null);
				
				// After transaction success we always try to redirect to
				// profile info view (not edit). Have to check if it works in all configurations... $$$
				$this->setRedirect(JRoute::_('index.php?option=com_users&view=profile&user_id=' . $userId, false));
			}
		}
		else
		{
			self::cleanUpUser($paymentData);
			$this->setMessage(JText::_('COM_SMARTSUB_PAYMENT_TRANSACTION_FAILED'), 'warning');
			if($paymentData->task == 'register')
			{
				$this->setRedirect(JRoute::_('index.php?option=com_users&view=registration', false));
			}
			else
			{
				$userId = $paymentData->user_id;
				$this->setRedirect(JRoute::_('index.php?option=com_users&view=profile&layout=edit&user_id=' . $userId, false));
			}
		}
		
		// flush payment data from session
		$app->setUserState('com_smartsub.payment_data', null);
		return true;
	}
	
	protected static function logTransaction($paymentData)
	{
		$payment_method = $paymentData->payment_method;
		
		if($paymentData->amount > 0)
		{
			// a real payment was made, so that log data is payment method specific
			$dispatcher = JDispatcher::getInstance();
			if(!JPluginHelper::importPlugin('smartsub', $payment_method))
			{
				return false;
			}
			
			$result = $dispatcher->trigger('onLogTransaction', array($paymentData));
			if (empty($result) || in_array(false, $result, true))
			{
				// Plugin will have to raise it's own error or throw an exception.
				return false;
			}
			
			$result = reset($result);
		}
		else 
		{
			// it was a free account upgrade
			$result = array(
				'user_id' => $paymentData->user_id,
				'amount' => $paymentData->amount,
				'currencyID' => $paymentData->currencyID,
				'status' => $paymentData->status,
				'transactionID' => SmartsubHelper::getInternalTransactionID($paymentData->username, $paymentData->id, 'GRATIS'),
				'order_time' => '',
				'system' => 'gratis',
				'offline' => false
			);
		}
		$result['user_id'] = $paymentData->user_id;
		$result['account_id'] = $paymentData->id;
		
		$result['discount'] = empty($paymentData->discount) ? 0 : $paymentData->discount;
		$result['upgrade_from'] = !empty($paymentData->upgrade_from) ? $paymentData->upgrade_from : 0;
		$result['date'] = JDate::getInstance()->toSql();
		
		// save current task
		$result['task'] = $paymentData->task;
		
		// load account data and calculate account expiration date
		JTable::addIncludePath(JPATH_COMPONENT_ADMINISTRATOR . '/tables');
		
		$db = JFactory::getDbo();
		
		// We have to select the latest payment of each type. The easiest way is to order
		// selected rows by date ascending and then "PHP-group" records, using
		// loadObjectList('status'): the rows selected last will win. This is not the
		// finest way, we have to check a quick and effective query where we can
		// order rows BEFORE grouping (not allowed as is in SQL)
		$db->setQuery('SELECT id, expires, status FROM #__ss_payments WHERE user_id = '
				. (int)$result['user_id'] . ' AND account_id = '
				. (int)$result['account_id'] . ' ORDER BY date ASC');
		try
		{
			$existing_payments = $db->loadObjectList('status');
		}
		catch(RuntimeException $e)
		{
			$existing_payments = array();
		}
		
		$existing_id = null;
		
		// get provisional expiration data for the account
		$expiration_data = SmartsubHelper::getExpirationDataById($result['account_id']);
		// "now" is set in getExpirationDataById(). Use it here as is, so that we discard
		// script execution time in future date comparisons
		$now_ts = $account_start_ts = $expiration_data['now_ts'];
		
		foreach($existing_payments as $status => $payment)
		{
			if($status == 'pending')
			{
				// there is a 'pending' payment row for this user and account, 
				// set $existing_id to update the row
				$existing_id = $payment->id;
			}
			if($status == 'paid' && $expiration_data['expires_ts'] > 0)
			{
				// if the user has already paid for the same account, 
				// we presume that he is renewing his account, so normally the new
				// expiration period must start on current expiration date
				$account_start_ts = max($now_ts, JDate::getInstance($payment->expires)->getTimestamp());
			}
		}
		
		if($expiration_data['expires_ts'] > 0)
		{
			// account has an expiration date. Check if we have to apply grace time - current
			// paid account valid time left (if any)
			
			// initially $account_start_ts = $now_ts, so $grace_time will be zero and expiration
			// correction will be also zero. Although, if $account_start_ts > $now_ts we will
			// apply this difference to provisional expiration time, always limiting by
			// expiration_alert_time (security measure)
			$grace_time = min($account_start_ts - $now_ts, $expiration_data['expiration_alert_time']);
			
			$expiration_data['expires_ts'] += $grace_time;
			$result['expires'] = JDate::getInstance($expiration_data['expires_ts'])->toSql();
		}
		else
		{
			// new account never expires
			$result['expires'] = $db->getNullDate();
		}
		
		// if we have a locked 'expires' date and time here, override the calculated value
		if(isset($paymentData->expires_locked))
		{
			$result['expires'] = $paymentData->expires_locked;
		}
		
		$payments = JTable::getInstance('Payment', 'SmartsubTable');
		
		// check if we already have a pending payment for the same user and account
		// if row exists, update it - do not create a new record
		$db->setQuery('SELECT id FROM #__ss_payments WHERE user_id = ' 
				. (int)$result['user_id'] . ' AND account_id = ' 
				. (int)$result['account_id'] . ' AND status = ' 
				. $db->Quote('pending') . ' ORDER BY date DESC LIMIT 1');
		try 
		{
			$existing_id = $db->loadResult();
		}
		catch(RuntimeException $e)
		{
			$existing_id = null;
		}
		
		if(!empty($existing_id))
		{
			$result['id'] = $existing_id;
		}
		
		if(!$payments->save($result))
		{
			// contact administrator!
			$this->setMessage(JText::_('COM_SMARTSUB_PAYMENT_LOGGING_FAILED'), 'warning');
			return false;
		}
		
		return $payments->id;
	}
	
	private static function cleanUpUser($paymentData)
	{
		if(empty($paymentData))
		{
			return;
		}
		$app = JFactory::getApplication();
		if($paymentData->task == 'register')
		{
			// new user payment error or user cancelled - delete the
			// user created (so that it will be possible to make more
			// attempts without duplicate username errors)
			$user = JUser::getInstance($paymentData->user_id);
			$user->delete();
			
			// also clear profile selection from session data
			// (it is required to get "pay" buttons appear)
			$data = $app->getUserState('com_users.registration.data');
			unset($data['smartsub']['account']);
			$app->setUserState('com_users.registration.data', $data);
		}
		else
		{
			// make sure we set current valid account into session data, so that 
			// profile view will display correct subscription after edit profile
			// is cancelled, but the user is still logged in and thus profile data
			// is read from session
			/*
			 * We may check that the code below is not required at all! $$$
			 */
			$data = $app->getUserState('com_users.edit.profile.data');
			$profile = JUserHelper::getProfile($paymentData->user_id);
			$account = !empty($profile->smartsub['account']) ? $profile->smartsub['account'] : null;
			$data['smartsub']['account'] = $account;
			$app->setUserState('com_users.edit.profile.data', $data);
		}
	}
	
	// this is specific for payment systems that callback in a new session
	// so that paymentData saved in session is not available.
	// Read payment data from request: *** MAKE SURE that you pass these custom
	// parameters on payment form/request to the payment server, so that they
	// can be read back here.
	private static function getPaymentDataFromRequest()
	{
		$app = JFactory::getApplication();
		$paymentData = new stdClass();
		
		$paymentData->payment_method = $app->input->getCmd('PR_PAYMENT_METHOD', '');
		$paymentData->user_id = $app->input->getInt('PR_USER_ID', 0);
		$paymentData->id = $app->input->getInt('PR_ACCOUNT_ID', 0);
		$paymentData->task = $app->input->getCmd('PR_TASK', '');
		// misc data. TODO: serialize all requred params to a url-encoded string $$$
		$paymentData->discount = $app->input->getCmd('PR_DISCOUNT', 0);
		
		if(empty($paymentData->payment_method))
		{
			$app->close();
		}
		
		return $paymentData;
	}
}
