<?php
/**
 * Plugin element to render dropdown list to select user
 *
 * @package     Joomla.Plugin
 * @subpackage  Fabrik.element.user
 * @copyright   Copyright (C) 2005-2013 fabrikar.com - All rights reserved.
 * @license     GNU/GPL http://www.gnu.org/copyleft/gpl.html
 */

// No direct access
defined('_JEXEC') or die('Restricted access');

require_once JPATH_SITE . '/plugins/fabrik_element/databasejoin/databasejoin.php';

/**
 * Plugin element to render dropdown list to select user
 *
 * @package     Joomla.Plugin
 * @subpackage  Fabrik.element.user
 * @since       3.0
 */

class PlgFabrik_ElementUser extends PlgFabrik_ElementDatabasejoin
{
	/**
	 * Db table field type
	 *
	 * @var string
	 */
	protected $fieldDesc = 'INT(11)';

	/**
	 * Load element params
	 * bit of a hack to set join_db_name in params
	 *
	 * @return  object  default element params
	 */

	public function getParams()
	{
		$params = parent::getParams();

		if (empty($params->join_db_name))
		{
			$params->set('join_db_name', '#__users');
		}
		// $$$ hugh - think we need to default key column as well, otherwise
		// when creating user element, we end up setting field to VARCHAR 255
		if (empty($params->join_key_column))
		{
			$params->set('join_key_column', 'id');
		}

		return $params;
	}

	/**
	 * Draws the html form element
	 *
	 * @param   array  $data           to pre-populate element with
	 * @param   int    $repeatCounter  repeat group counter
	 *
	 * @return  string	elements html
	 */

	public function render($data, $repeatCounter = 0)
	{
		$element = $this->getElement();
		$name = $this->getHTMLName($repeatCounter);
		$html_id = $this->getHTMLId($repeatCounter);
		$id = $html_id;
		$params = $this->getParams();

		/**
		 *  $$$ rob - if embedding a form inside a details view then rowid is true (for the detailed view) but we are still showing a new form
		 *  so take a look at the element form's rowId and not app input
		 */
		$rowid = $this->getFormModel()->rowId;
		/**
		 * @TODO when editing a form with joined repeat group the rowid will be set but
		 * the record is in fact new
		 */
		if ($params->get('update_on_edit') || !$rowid || ($this->inRepeatGroup && $this->_inJoin && $this->_repeatGroupTotal == $repeatCounter))
		{
			// Set user to logged in user
			if ($this->isEditable())
			{
				$user = JFactory::getUser();
			}
			else
			{
				$userid = (int) $this->getValue($data, $repeatCounter);

				// On failed validation value is 1 - user ids are always more than that so don't load userid=1 otherwise an error is generated
				$user = $userid <= 1 ? JFactory::getUser() : JFactory::getUser($userid);
			}
		}
		else
		{
			/**
			 *  $$$ hugh - this is blowing away the userid, as $element->default is empty at this point
			 *  so for now I changed it to the $data value
			 *  keep previous user
			 *  $user = JFactory::getUser((int) $element->default);
			 */
			// $$$ hugh ... what a mess ... of course if it's a new form, $data doesn't exist ...
			if (empty($data))
			{
				$user = JFactory::getUser();
			}
			else
			{
				if ($this->inDetailedView)
				{
					$id = preg_replace('#_ro$#', '_raw', $id);
				}
				else
				{
					/**
					 *  $$$ rob 31/07/2011 not sure this is right - causes js error when field is hidden in form
					 *  $$$ hugh 10/31/2011 - but if we don't do it, $id is the label not the value (like 'username')
					 *  so wrong uid is written to form, and wipes out real ID when form is submitted.
					 *  OK, problem was we were using $id further on as the html ID, so if we added _raw, element
					 *  on form had wrong ID.  Added $html_id above, to use as (duh) html ID instead of $id.
					 */
					if (!strstr($id, '_raw') && array_key_exists($id . '_raw', $data))
					{
						$id .= '_raw';
					}
				}

				$id = FArrayHelper::getValue($data, $id, '');

				if ($id === '')
				{
					$id = $this->getValue($data, $repeatCounter);
				}

				/*
				 * After a failed validation, it may be JSON, and urlencoded, like [&quot;94&quot;]
				 * Or it may be an array with JSON and or urlencode and or ... yada yada ... who the f*ck knows
				 * So let's just cover all the bases, shall we?
				 */
				
				$id = is_array($id) ? $id[0] : $id;
				
				$id = html_entity_decode($id);
				if (FabrikWorker::isJSON($id))
				{
					$id = FabrikWorker::JSONtoData($id, true);
				}

				$id = is_array($id) ? $id[0] : $id;
				/* $$$ hugh - hmmm, might not necessarily be a new row.  So corner case check for
				 * editing a row, where user element is not set yet, and 'update on edit' is No.
				 */
				if ($rowid && empty($id) && !$params->get('update_on_edit'))
				{
					$user = JFactory::getUser(0);
				}
				else
				{
					$user = $id === '' ? JFactory::getUser() : JFactory::getUser((int) $id);
				}
			}
		}

		$displayParam = $this->getLabelOrConcatVal();
		$layout = $this->getLayout('form');
		$layoutData = new stdClass;
		$layoutData->inJDb = $this->inJDb();
		$layoutData->name = $name;
		$layoutData->id = $html_id;
		$layoutData->isEditable = $this->isEditable();
		$layoutData->hidden = $element->hidden;
		$layoutData->input = parent::render($data, $repeatCounter);
		$layoutData->readOnly = is_a($user, 'JUser') ? $user->get($displayParam) : '';
		$layoutData->value = is_a($user, 'JUser') ? $user->get('id') : '';

		return $layout->render($layoutData);
	}

	/**
	 * Is the element hidden or not - if not set then return false
	 * If the table db isn't the same as the joomla db the element
	 *
	 * @return  bool
	 */

	public function isHidden()
	{
		if ($this->inJDb())
		{
			return parent::isHidden();
		}
		else
		{
			return true;
		}
	}

	/**
	 * run on formModel::setFormData()
	 *
	 * @param   int  $c  repeat group counter
	 *
	 * @return void
	 */

	public function preProcess($c)
	{
		$app = JFactory::getApplication();
		$input = $app->input;
		$params = $this->getParams();

		/**
		 * $$$ hugh - special case for social plugins (like CB plugin).  If plugin sets
		 * fabrik.plugin.profile_id, and 'user_use_social_plugin_profile' param is set,
		 * and we are creating a new row, then use the session data as the user ID.
		 * This allows user B to view a table in a CB profile for user A, do an "Add",
		 * and have the user element set to user A's ID.
		 * TODO - make this table/form specific, but not so easy to do in CB plugin
		 */
		if ((int) $params->get('user_use_social_plugin_profile', 0))
		{
			if ($input->getString('rowid', '', 'string') == '' && $input->get('task') !== 'doimport')
			{
				$context = 'fabrik.plugin.profile_id';

				if ($input->get('fabrik_social_profile_hash', '') != '')
				{
					$context = 'fabrik.plugin.' . $input->get('fabrik_social_profile_hash', '') . '.profile_id';
				}

				$session = JFactory::getSession();

				if ($session->has($context))
				{
					$profile_id = $session->get($context);
					$form = $this->getFormModel();
					$group = $this->getGroup();
					$joinid = $group->getGroup()->join_id;
					$key = $this->getFullName(true, false);
					$shortkey = $this->getFullName(true, false);
					$rawkey = $key . '_raw';

					if ($group->canRepeat())
					{
						if ($group->isJoin())
						{
							$key = str_replace("][", '.', $key);
							$key = str_replace(array('[', ']'), '.', $key) . "$c";
							$rawkey = str_replace($shortkey, $shortkey . '_raw', $key);
						}
						else
						{
							$key = $key . '.' . $c;
							$rawkey = $rawkey . '.' . $c;
						}
					}
					else
					{
						if ($group->isJoin())
						{
							$key = str_replace("][", ".", $key);
							$key = str_replace(array('[', ']'), '.', $key);
							$key = rtrim($key, '.');
							$rawkey = str_replace($shortkey, $shortkey . '_raw', $key);
						}
					}

					$form->updateFormData($key, $profile_id);
					$form->updateFormData($rawkey, $profile_id);
					$input->post->set($key, $profile_id);
					$input->post->set($rawkey, $profile_id);
				}
			}
		}
	}

	/**
	 * Trigger called when a row is stored.
	 * If we are creating a new record, and the element was set to readonly
	 * then insert the users data into the record to be stored
	 *
	 * @param   array  &$data          Data to store
	 * @param   int    $repeatCounter  Repeat group index
	 *
	 * @return  bool  If false, data should not be added.
	 */

	public function onStoreRow(&$data, $repeatCounter = 0)
	{
		if (!parent::onStoreRow($data, $repeatCounter))
		{
			return false;
		}

		$app = JFactory::getApplication();
		$input = $app->input;

		// $$$ hugh - special case, if we have just run the fabrikjuser plugin, we need to
		// use the 'newuserid' as set by the plugin.
		$newuserid = $input->getInt('newuserid', 0);

		if (!empty($newuserid))
		{
			$newuserid_element = $input->get('newuserid_element', '');
			$this_fullname = $this->getFullName(true, false);

			if ($newuserid_element == $this_fullname)
			{
				return true;
			}
		}

		$element = $this->getElement();
		$params = $this->getParams();

		/*
		 * After a failed validation, if readonly for ACL's, it may be JSON, and urlencoded, like [&quot;94&quot;]
		*/

		$data[$element->name] = is_array($data[$element->name]) ? $data[$element->name][0] : $data[$element->name];

		$data[$element->name] = html_entity_decode($data[$element->name]);

		if (FabrikWorker::isJSON($data[$element->name]))
		{
			$data[$element->name] = FabrikWorker::JSONtoData($data[$element->name], true);
		}

		$data[$element->name] = is_array($data[$element->name]) ? $data[$element->name][0] : $data[$element->name];


		/**
		 *  $$$ hugh - special case for social plugins (like CB plugin).  If plugin sets
		 *  fabrik.plugin.profile_id, and 'user_use_social_plugin_profile' param is set,
		 *  and we are creating a new row, then use the session data as the user ID.
		 * This allows user B to view a table in a CB profile for user A, do an "Add",
		 * and have the user element set to user A's ID.
		 */
		// TODO - make this table/form specific, but not so easy to do in CB plugin
		if ((int) $params->get('user_use_social_plugin_profile', 0))
		{
			//if ($input->getString('rowid', '', 'string') == '' && $input->get('task') !== 'doimport')
			if ($input->getString('rowid', '', 'string') == '' && !$this->getListModel()->importingCSV)
			{
				$session = JFactory::getSession();

				if ($session->has('fabrik.plugin.profile_id'))
				{
					$data[$element->name] = $session->get('fabrik.plugin.profile_id');
					$data[$element->name . '_raw'] = $data[$element->name];

					// $session->clear('fabrik.plugin.profile_id');
					return true;
				}
			}
		}

		// $$$ rob also check we aren't importing from CSV - if we are ignore
		//if ($input->getString('rowid', '', 'string') == '' && $input->get('task') !== 'doimport')
		if ($input->getString('rowid', '', 'string') == '' && !$this->getListModel()->importingCSV)
		{
			// $$$ rob if we cant use the element or its hidden force the use of current logged in user
			if (!$this->canUse() || $this->getElement()->hidden == 1)
			{
				$user = JFactory::getUser();
				$data[$element->name] = $user->get('id');
				$data[$element->name . '_raw'] = $data[$element->name];
			}
		}
		// $$$ hugh
		// If update-on-edit is set, we always want to store as current user??

		// $$$ rob NOOOOOO!!!!! - if its HIDDEN OR set to READ ONLY then yes
		// otherwise selected dropdown option is not taken into account

		// $$$ hugh - so how come we don't do the same thing on a new row?  Seems inconsistent to me?

		// $$$ paul - seems bonkers to me to use source code comments like an instant messaging system!

		/**
		 * $$$ hugh - it's not IM'ing, it's long running "frank and honest differences of opinion" over how things work
		 * and why we each make the assumptions / changes we do when working on "disputed" chunks of code
		 */

		else
		{
			if ($this->updateOnEdit())
			{
				$user = JFactory::getUser();
				$data[$element->name] = $user->get('id');
				$data[$element->name . '_raw'] = $data[$element->name];

				// $$$ hugh - need to add to updatedByPlugin() in order to override write access settings.
				// This allows us to still 'update on edit' when element is write access controlled.
				if (!$this->canUse())
				{
					$this_fullname = $this->getFullName(true, false);
					$this->getFormModel()->updatedByPlugin($this_fullname, $user->get('id'));
				}
			}

			/**
			 * If importing from CSV and not set to update on edit, let's check to see if they
			 * are trying to import a username rather than ID.
			 */

			else if ($this->getListModel()->importingCSV)
			{
				$formData = $this->getFormModel()->formData;
				$userid = FArrayHelper::getValue($formData, $element->name, '');
				if (!empty($userid) && !is_numeric($userid))
				{
					$user = JFactory::getUser($userid);
					$new_userid = $user->get('id');
					if (empty($new_userid) && FabrikWorker::isEmail($userid))
					{
						$db = JFactory::getDbo();
						$query = $db->getQuery(true)
						->select($db->quoteName('id'))
						->from($db->quoteName('#__users'))
						->where($db->quoteName('email') . ' = ' . $db->quote($userid));
						$db->setQuery($query, 0, 1);

						$new_userid = (int) $db->loadResult();
					}
					$data[$element->name] = $new_userid;
				}
			}
		}

		return true;
	}

	/**
	 * Should the element's value be replaced with the current user's id
	 *
	 * @return  bool
	 */
	protected function updateOnEdit()
	{
		$params = $this->getParams();
		$updaeOnEdit = $params->get('update_on_edit', 0);

		if ($updaeOnEdit == 1)
		{
			$updaeOnEdit = !$this->canUse() || $this->getElement()->hidden == 1;
		}

		if ($updaeOnEdit == 2)
		{
			$updaeOnEdit = true;
		}

		return $updaeOnEdit;
	}

	/**
	 * Check user can view the read only element OR view in list view
	 *
	 * When processing the form, we always want to store the current userid
	 * (subject to save-on-edit, but that's done elsewhere), regardless of
	 * element access settings, see:
	 *
	 * http://fabrikar.com/forums/showthread.php?p=70554#post70554
	 *
	 * So overriding the element model canView and returning true in that
	 * case allows addDefaultDataFromRO to do that, whilst still enforcing
	 * Read Access settings for detail/list view
	 *
	 * @param   string  $view  View list/form @since 3.0.7
	 *
	 * @return  bool  can view or not
	 */

	public function canView($view = 'form')
	{
		$app = JFactory::getApplication();

		if ($app->input->get('task', '') == 'processForm')
		{
			return true;
		}

		return parent::canView($view);
	}

	/**
	 * Returns javascript which creates an instance of the class defined in formJavascriptClass()
	 *
	 * @param   int  $repeatCounter  Repeat group counter
	 *
	 * @return  array
	 */

	public function elementJavascript($repeatCounter)
	{
		$opts = parent::elementJavascriptOpts($repeatCounter);
		$id = $this->getHTMLId($repeatCounter);

		return array('FbUser', $id, $opts);
	}

	/**
	 * Get the class to manage the form element
	 * to ensure that the file is loaded only once
	 *
	 * @param   array   &$srcs   Scripts previously loaded
	 * @param   string  $script  Script to load once class has loaded
	 * @param   array   &$shim   Dependant class names to load before loading the class - put in requirejs.config shim
	 *
	 * @return void
	 */

	public function formJavascriptClass(&$srcs, $script = '', &$shim = array())
	{
		$s = new stdClass;
		$s->deps = array('fab/element');
		$shim['element/databasejoin/databasejoin'] = $s;

		$s = new stdClass;
		$s->deps = array('element/databasejoin/databasejoin');
		$shim['element/user/user'] = $s;

		parent::formJavascriptClass($srcs, $script, $shim);
	}

	/**
	 * Get select option label
	 *
	 * @param  bool  $filter  get alt label for filter, if present using :: splitter
	 *
	 * @return  string
	 */

	protected function _getSelectLabel($filter = false)
	{
		return $this->getParams()->get('user_noselectionlabel', FText::_('COM_FABRIK_PLEASE_SELECT'));
	}

	/**
	 * Create the SQL select 'name AS alias' segment for list/form queries
	 *
	 * @param   array  &$aFields    array of element names
	 * @param   array  &$aAsFields  array of 'name AS alias' fields
	 * @param   array  $opts        options
	 *
	 * @return  void
	 */

	public function getAsField_html(&$aFields, &$aAsFields, $opts = array())
	{
		$table = $this->actualTableName();
		$element = $this->getElement();
		$params = $this->getParams();
		$db = FabrikWorker::getDbo();
		$fullElName = FArrayHelper::getValue($opts, 'alias', $table . '___' . $element->name);

		// Check if main database is the same as the elements database
		if ($this->inJDb())
		{
			/**
			 * it is so continue as if it were a database join
			 * make sure same connection as this table
			 */

			$join = $this->getJoin();

			// $$$ rob in csv import keytable not set
			$k = isset($join->keytable) ? $join->keytable : $join->join_from_table;
			$k = FabrikString::safeColName($k . '.' . $element->name);
			$k2 = FabrikString::safeColName($this->getJoinLabelColumn());

			if (FArrayHelper::getValue($opts, 'inc_raw', true))
			{
				$aFields[] = $k . ' AS ' . $db->quoteName($fullElName . '_raw');
				$aAsFields[] = $db->quoteName($fullElName . '_raw');
			}

			$aFields[] = $k2 . ' AS ' . $db->quoteName($fullElName);
			$aAsFields[] = $db->quoteName($fullElName);
		}
		else
		{
			$k = $db->quoteName($table) . '.' . $db->quoteName($element->name);

			// Its not so revert back to selecting the id
			$aFields[] = $k . ' AS ' . $db->quoteName($fullElName . '_raw');
			$aAsFields[] = $db->quoteName($fullElName . '_raw');
			$aFields[] = $k . ' AS ' . $db->quoteName($fullElName);
			$aAsFields[] = $db->quoteName($fullElName);
		}
	}

	/**
	 * Called when the element is saved
	 *
	 * @param   array  $data  posted element save data
	 *
	 * @return  bool  save ok or not
	 */

	public function onSave($data)
	{
		$params = json_decode($data['params']);

		if (!$this->canEncrypt() && !empty($params->encrypt))
		{
			throw new InvalidArgumentException('The encryption option is only available for field and text area plugins');
		}

		$label = (isset($params->my_table_data) && $params->my_table_data !== '') ? $params->my_table_data : 'username';
		$this->updateFabrikJoins($data, '#__users', 'id', $label);

		return true;
	}

	/**
	 * Get the join label name
	 *
	 * @return  string
	 */

	protected function getJoinLabel()
	{
		$label = parent::getJoinLabel();

		if ($label == 'gid')
		{
			$label = 'username';
		}

		return $label;
	}

	/**
	 * This really does get just the default value (as defined in the element's settings)
	 *
	 * @param   array  $data  Form data
	 *
	 * @return mixed
	 */

	public function getDefaultValue($data = array())
	{
		if (!isset($this->default))
		{
			$user = JFactory::getUser();
			$this->default = $user->get('id');
		}

		return $this->default;
	}

	/**
	 * Determines the value for the element in the form view
	 *
	 * @param   array  $data           form data
	 * @param   int    $repeatCounter  when repeating joined groups we need to know what part of the array to access
	 * @param   array  $opts           options
	 *
	 * @return  string	value
	 */

	public function getValue($data, $repeatCounter = 0, $opts = array())
	{
		$app = JFactory::getApplication();
		$input = $app->input;

		// Kludge for 2 scenarios
		if (array_key_exists('rowid', $data))
		{
			// When validating the data on form submission
			$key = 'rowid';
		}
		else
		{
			// When rendering the element to the form
			$key = '__pk_val';
		}
		/*
		 * empty(data) when you are saving a new record and this element is in a joined group
		 * $$$ hugh - added !array_key_exists(), as ... well, rowid doesn't always exist in the query string
		 */

		if (empty($data) || !array_key_exists($key, $data))
		{
			// $$$ rob - added check on task to ensure that we are searching and not submitting a form
			// as otherwise not empty validation failed on user element
			if (!in_array($input->get('task'), array('processForm', 'view', '', 'form.process', 'process')))
			{
				return '';
			}

			return $this->getDefaultOnACL($data, $opts);
		}

		return parent::getValue($data, $repeatCounter, $opts);
	}

	/**
	 * Get the table filter for the element
	 *
	 * @param   int   $counter  filter order
	 * @param   bool  $normal   do we render as a normal filter or as an advanced search filter
	 * if normal include the hidden fields as well (default true, use false for advanced filter rendering)
	 *
	 * @return  string	filter html
	 */

	public function getFilter($counter = 0, $normal = true)
	{
		$listModel = $this->getlistModel();
		$formModel = $listModel->getFormModel();
		$elName2 = $this->getFullName(false, false);

		if (!$formModel->hasElement($elName2))
		{
			return '';
		}

		$table = $listModel->getTable();
		$element = $this->getElement();
		$params = $this->getParams();

		$elName = $this->getFullName(true, false);
		$htmlid = $this->getHTMLId() . 'value';
		$v = $this->filterName($counter, $normal);

		// Correct default got
		$default = $this->getDefaultFilterVal($normal, $counter);
		$return = array();
		$tabletype = $this->getLabelOrConcatVal();
		$join = $this->getJoin();
		$joinTableName = FabrikString::safeColName($join->table_join_alias);

		// If filter type isn't set was blowing up in switch below 'cos no $rows
		// so added '' to this test.  Should probably set $element->filter_type to a default somewhere.
		if (in_array($element->filter_type, array('range', 'dropdown', '', 'checkbox')))
		{
			$rows = $this->filterValueList($normal, '', $joinTableName . '.' . $tabletype, '', false);
			$rows = (array) $rows;

			if ($element->filter_type !== 'checkbox')
			{
				array_unshift($rows, JHTML::_('select.option', '', $this->filterSelectLabel()));
			}
		}

		$class = $this->filterClass();

		switch ($element->filter_type)
		{
			case 'checkbox':
				$return[] = $this->checkboxFilter($rows, $default, $v);
				break;
			case "range":
				$attribs = 'class="' . $class . '" size="1" ';
				$default1 = is_array($default) ? $default[0] : '';
				$return[] = JHTML::_('select.genericlist', $rows, $v . '[]', $attribs, 'value', 'text', $default1, $element->name . "_filter_range_0");
				$default1 = is_array($default) ? $default[1] : '';
				$return[] = JHTML::_('select.genericlist', $rows, $v . '[]', $attribs, 'value', 'text', $default1, $element->name . "_filter_range_1");
				break;
			case 'dropdown':
			case 'multiselect':
			default:
				$max = count($rows) < 7 ? count($rows) : 7;
				$size = $element->filter_type === 'multiselect' ? 'multiple="multiple" size="' . $max . '"' : 'size="1"';
				$v = $element->filter_type === 'multiselect' ? $v . '[]' : $v;
				$return[] = JHTML::_('select.genericlist', $rows, $v, 'class="' . $class . '" ' . $size, 'value', 'text', $default, $htmlid);
				break;

			case 'field':
				if (get_magic_quotes_gpc())
				{
					$default = stripslashes($default);
				}

				$default = htmlspecialchars($default);
				$return[] = '<input type="text" name="' . $v . '" class="' . $class . '" value="' . $default . '" id="' . $htmlid . '" />';
				break;

			case 'hidden':
				if (get_magic_quotes_gpc())
				{
					$default = stripslashes($default);
				}

				$default = htmlspecialchars($default);
				$return[] = '<input type="hidden" name="' . $v . '" class="' . $class . '" value="' . $default . '" id="' . $htmlid . '" />';
				break;

			case 'auto-complete':
				$defaultLabel = $this->getLabelForValue($default);
				$autoComplete = $this->autoCompleteFilter($default, $v, $defaultLabel, $normal);
				$return = array_merge($return, $autoComplete);
				break;
		}

		if ($normal)
		{
			$return[] = $this->getFilterHiddenFields($counter, $elName, false, $normal);
		}
		else
		{
			$return[] = $this->getAdvancedFilterHiddenFields();
		}

		return implode("\n", $return);
	}

	/**
	 * If filterValueList_Exact incjoin value = false, then this method is called
	 * to ensure that the query produced in filterValueList_Exact contains at least the database join element's
	 * join
	 *
	 * @return  string  required join text to ensure exact filter list code produces a valid query.
	 */

	protected function buildFilterJoin()
	{
		$params = $this->getParams();
		$joinTable = FabrikString::safeColName($params->get('join_db_name'));
		$join = $this->getJoin();
		$joinTableName = FabrikString::safeColName($join->table_join_alias);
		$joinKey = $this->getJoinValueColumn();
		$elName = FabrikString::safeColName($this->getFullName(true, false));

		return 'INNER JOIN ' . $joinTable . ' AS ' . $joinTableName . ' ON ' . $joinKey . ' = ' . $elName;
	}

	/**
	 * Build the filter query for the given element.
	 * Can be overwritten in plugin - e.g. see checkbox element which checks for partial matches
	 *
	 * @param   string  $key            element name in format `tablename`.`elementname`
	 * @param   string  $condition      =/like etc.
	 * @param   string  $value          search string - already quoted if specified in filter array options
	 * @param   string  $originalValue  original filter value without quotes or %'s applied
	 * @param   string  $type           filter type advanced/normal/prefilter/search/querystring/searchall
	 *
	 * @return  string	sql query part e,g, "key = value"
	 */

	public function getFilterQuery($key, $condition, $value, $originalValue, $type = 'normal')
	{
		if (!$this->inJDb())
		{
			return $key . ' ' . $condition . ' ' . $value;
		}

		$element = $this->getElement();
		$db = JFactory::getDbo();

		// $$$ hugh - we need to use the join alias, not hard code #__users
		$join = $this->getJoin();

		if (is_object($join))
		{
			$joinTableName = $join->table_join_alias;
		}

		if (empty($joinTableName))
		{
			$joinTableName = '#__users';
		}

		if ($type == 'querystring' || $type == 'jpluginfilters')
		{
			$key = FabrikString::safeColNameToArrayKey($key);
			/* $$$ rob no matter whether you use elementname_raw or elementname in the querystring filter
			 * by the time it gets here we have normalized to elementname. So we check if the original qs filter was looking at the raw
			 * value if it was then we want to filter on the key and not the label
			 */
			$filter = JFilterInput::getInstance();
			$get = $filter->clean($_GET, 'array');

			if (!array_key_exists($key, $get))
			{
				$key = $db->quoteName($joinTableName . '.id');
				$this->encryptFieldName($key);

				return $key . ' ' . $condition . ' ' . $value;
			}
		}

		if ($type == 'advanced')
		{
			$key = $db->quoteName($joinTableName . '.id');
			$this->encryptFieldName($key);

			return $key . ' ' . $condition . ' ' . $value;
		}

		$params = $this->getParams();

		if ($type != 'prefilter')
		{
			switch ($element->filter_type)
			{
				case 'range':
				case 'dropdown':
					$tabletype = 'id';
					break;
				case 'field':
				default:
					$tabletype = $this->getLabelOrConcatVal();
					break;
			}

			$k = $db->quoteName($joinTableName . '.' . $tabletype);
		}
		else
		{
			if ($this->_rawFilter)
			{
				$k = $db->quoteName($joinTableName . '.id');
			}
			else
			{
				$tabletype = $this->getLabelOrConcatVal();
				$k = $db->quoteName($joinTableName . '.' . $tabletype);
			}
		}

		$this->encryptFieldName($k);
		$str = $k . ' ' . $condition . ' ' . $value;

		return $str;
	}

	/**
	 * Get the database object
	 *
	 * @return  object	database
	 */

	public function getDb()
	{
		return FabrikWorker::getDbo(true);
	}

	/**
	 * Used to format the data when shown in the form's email
	 *
	 * @param   mixed  $value          element's data
	 * @param   array  $data           form records data
	 * @param   int    $repeatCounter  repeat group counter
	 *
	 * @return  string	formatted value
	 */

	public function getEmailValue($value, $data = array(), $repeatCounter = 0)
	{
		$key = $this->getFullName(true, false);
		$rawkey = $key . '_raw';
		$userid = $value;

		if (array_key_exists($rawkey, $data))
		{
			$userid = $data[$rawkey];
		}
		elseif (array_key_exists($key, $data))
		{
			$userid = $data[$key];
		}

		if ($this->getGroup()->canRepeat())
		{
			$userid = FArrayHelper::getValue($userid, $repeatCounter, 0);
		}
		
		if (is_array($userid))
		{
			$userid = (int) array_shift($userid);
		}
		else
		{
			// Test json string e.g. ["350"] - fixes JUser: :_load: User does not exist notices
			if (!is_int($userid))
			{
				$userid = FabrikWorker::JSONtoData($userid, true);
				$userid = (int) FArrayHelper::getValue($userid, 0, 0);
			}
		}

		$user = $userid === 0 ? JFactory::getUser() : JFactory::getUser($userid);

		return $this->getUserDisplayProperty($user);
	}

	/**
	 * Get the user's property to show, if gid raise warning and revert to username (no gid in J1.7)
	 *
	 * @param   object  $user  Joomla user
	 *
	 * @since	3.0b
	 *
	 * @return  string
	 */

	protected function getUserDisplayProperty($user)
	{
		$displayParam = $this->getLabelOrConcatVal();

		return is_a($user, 'JUser') ? $user->get($displayParam) : false;
	}

	/**
	 * Get the column name used for the value part of the db join element
	 *
	 * @return  string
	 */

	protected function getJoinValueColumn()
	{
		$join = $this->getJoin();
		$db = FabrikWorker::getDbo();

		return $db->quoteName($join->table_join_alias) . '.id';
	}

	/**
	 * Used for the name of the filter fields
	 * Over written here as we need to get the label field for field searches
	 *
	 * @return string element filter name
	 */

	public function getFilterFullName()
	{
		$elName = $this->getFullName(true, false);

		return FabrikString::safeColName($elName);
	}

	/**
	 * Called when copy row list plugin called
	 *
	 * @param   mixed  $val  value to copy into new record
	 *
	 * @return mixed value to copy into new record
	 */

	public function onCopyRow($val)
	{
		$params = $this->getParams();

		if ($params->get('update_on_edit'))
		{
			$user = JFactory::getUser();
			$val = $user->get('id');
		}

		return $val;
	}

	/**
	 * Called when save as copy form button clicked
	 *
	 * @param   mixed  $val  value to copy into new record
	 *
	 * @return  mixed  value to copy into new record
	 */

	public function onSaveAsCopy($val)
	{
		$params = $this->getParams();

		if ($params->get('update_on_copy', false))
		{
			$user = JFactory::getUser();
			$val = $user->get('id');
		}

		return $val;
	}

	/**
	 * Get the element name or concat statement used to build the dropdown labels or
	 * table data field
	 *
	 * @return  string
	 */

	protected function getLabelOrConcatVal()
	{
		static $displayMessage;
		$app = JFactory::getApplication();
		$params = $this->getParams();
		$displayParam = $params->get('my_table_data', 'username');

		if ($displayParam == 'gid')
		{
			$displayParam == 'username';

			if (!isset($displayMessage))
			{
				$app->enqueueMessage(JText::sprintf('PLG_ELEMENT_USER_NOTICE_GID', $this->getElement()->id), 'notice');
				$displayMessage = true;
			}
		}

		return $displayParam;
	}

	/**
	 * Get an array of element html ids and their corresponding
	 * js events which trigger a validation.
	 * Examples of where this would be overwritten include timedate element with time field enabled
	 *
	 * @param   int  $repeatCounter  repeat group counter
	 *
	 * @return  array  html ids to watch for validation
	 */

	public function getValidationWatchElements($repeatCounter)
	{
		$id = $this->getHTMLId($repeatCounter);
		$ar = array('id' => $id, 'triggerEvent' => 'change');

		return array($ar);
	}
}
