<?php
/**
 * @package plg_smartsub_offline (Offline payment plugin for Smart Subscriptions) for Joomla 3
 * @version 1.0
 * @author Alex Polonski
 * @copyright (C)2014 - Alex Polonski
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 **/
// No direct access.
defined('_JEXEC') or die;

if(file_exists(JPATH_ADMINISTRATOR . '/components/com_smartsub/helpers/smartsub.php'))
{
	require_once(JPATH_ADMINISTRATOR . '/components/com_smartsub/helpers/payment.php');
	require_once(JPATH_ADMINISTRATOR . '/components/com_smartsub/helpers/smartsub.php');
}

class plgSmartsubOffline extends PaymentPlugin
{
	
	public function __construct(&$subject, $config)
	{
		parent::__construct($subject, $config);
	
		$this->payment_method = 'offline';
		$this->is_offline = 1;
		
		JFactory::getLanguage()->load('plg_smartsub_offline', dirname(__FILE__));
	}
	
	protected function getApiCredentials()
	{
		$result = new stdClass();
		
		$app = JFactory::getApplication();
		
		$result->beneficiary = $this->params->get('beneficiary', $app->get('sitename'));
		$result->bank = $this->params->get('bank', '');
		$result->address1 = $this->params->get('address1', '');
		$result->address2 = $this->params->get('address2', '');
		$result->account = $this->params->get('account', '');
		$result->swift = $this->params->get('swift', '');
		$result->code = $this->params->get('code', '');
		$result->iban = $this->params->get('iban', '');

		return $result;
	}
	
	public function onGetPaymentLinkData()
	{
		if(!$this->componentInstalled)
		{
			return false;
		}
		
		$result = array(
				'payment_method' => $this->payment_method,
				// we use a dedicated method here
				'html' => $this->getModalHTML(),
				'action' => JRoute::_('index.php?option=com_smartsub&task=payment.init&payment_method=' . $this->payment_method, false)
		);
		
		JHtml::_('bootstrap.modal', 'pr-payment-method-offline', array('show' => 0));
		
		$doc = JFactory::getDocument();
		$doc->addScript(JUri::root() . 'plugins/smartsub/offline/offline.js');
		
		$credentials = $this->getApiCredentials();
		$options = array(
				'modal_id' => 'pr-offline-modal',
				'concept_span' => 'pr-offline-info-concept',
				'amount_span' => 'pr-offline-info-amount',
				'username_field_id' => 'jform_username',
				'transactionID_pattern' => $this->getTransactionIDpattern()
		);
		$doc->addScriptDeclaration("
			(function ($){
				$(document).ready(function (){
 					pr_offline_account_data.init(" . json_encode($options) . ")
 				});
			})(jQuery);
			"
		);
		
		return $result;
	}
	
	public function onInitPayment($paymentData = null)
	{
		if(!$this->componentInstalled)
		{
			return false;
		}
		
		return array(
				'payment_data' => $paymentData,
				// we can skip 'getdetails' task for offline payments, go to dotransaction directly
				'redirect_url' => JRoute::_('index.php?option=com_smartsub&task=payment.dotransaction', false)
		);
	}
	
	public function onGetDetails($paymentData = null)
	{
		// never called
	}
	
	public function onDoTransaction($paymentData = null)
	{
		if(!$this->componentInstalled)
		{
			return false;
		}
		
		$paymentData->complete = true;
		$paymentData->status = 'pending';
		return array(
				'payment_data' => $paymentData,
				'redirect_url' => ''
		);
	}
	
	public function onLogTransaction($paymentData = null)
	{
		if(!$this->componentInstalled)
		{
			return false;
		}
		
		if($paymentData->complete)
		{
			$app = JFactory::getApplication();
			
			$transactionID = sprintf($this->getTransactionIDpattern(), strtoupper($paymentData->username), $paymentData->id);
				
			$paymentData->transactionID = $transactionID;
			
			$body = SmartsubHelper::processEmailTpl($paymentData, 'offline');
			
			// inject bank data, set subject and send
			$body = str_replace('{%bankdata}', $this->getEmailHTML($paymentData), $body);
			$subject = JText::sprintf('PLG_SMARTSUB_OFFLINE_EMAIL_SUBJECT', $transactionID);
			$sent = SmartsubHelper::sendNotification($paymentData->user_id, $subject, $body);
			
			// return main transaction details in the format common for all payment plugins
			return array(
				'user_id' => $paymentData->user_id,
				'amount' => $paymentData->amount,
				'currencyID' => $paymentData->currencyID,
				'status' => $paymentData->status,
				'transactionID' => $transactionID,
				'order_time' => '',
				'system' => $this->payment_method,
				'offline' => $this->is_offline
			);
		}
		return false;
	}
	
	private function getEmailHTML($paymentData)
	{
		$options = $this->getApiCredentials();
		$transactionID = sprintf($this->getTransactionIDpattern(), strtoupper($paymentData->username), $paymentData->id);

		ob_start();
?>
		<table>
			<tbody>
			<?php foreach($options as $key => $value) : ?>
				<?php if(!empty($value)) : ?>
				<tr>
					<th align="left"><?php echo JText::_('PLG_SMARTSUB_OFFLINE_' . $key); ?></th>
					<td><?php echo htmlspecialchars($value); ?></td>
				<tr>
				<?php endif; ?>
			<?php endforeach; ?>
				<tr>
					<th align="left"><?php echo JText::_('PLG_SMARTSUB_OFFLINE_CONCEPT'); ?></th>
					<td><?php echo htmlspecialchars($transactionID); ?></td>
				</tr>
				<tr>
					<th align="left"><?php echo JText::_('PLG_SMARTSUB_OFFLINE_AMOUNT'); ?></th>
					<td><?php echo htmlspecialchars(SmartsubHelper::formatPrice($paymentData->amount, $paymentData->currencyID)); ?></td>
				</tr>
			</tbody>
		</table>
<?php		
		$content = ob_get_clean();
		return $content;
	}
	
	private function getModalHTML()
	{
		$options = $this->getApiCredentials();
		ob_start();
?>
		<input data-target="#pr-offline-modal" id="pr-payment-method-offline" data-payment-method="<?php echo $this->payment_method; ?>" onclick="pr_offline_account_data.showModal(this)" class="action-payment btn btn-primary" type="button" value="<?php echo JText::_('PLG_SMARTSUB_OFFLINE_BUTTON_TEXT'); ?>" />
		<div id="pr-offline-modal" class="modal hide fade" tabindex="-1" role="dialog" aria-hidden="true"  aria-labelledby="pr-offline-info-label">
			<div class="modal-header">
				<button type="button" class="close" data-dismiss="modal" aria-hidden="true">×</button>
				<h3 id="pr-offline-info-label"><?php echo JText::_('PLG_SMARTSUB_OFFLINE_MODAL_HEADER'); ?></h3>
			</div>
			<div class="modal-body">
				<table class="table">
					<thead>
						<tr>
							<th colspan="2"><?php echo JText::_('PLG_SMARTSUB_OFFLINE_INSTRUCTIONS'); ?></th>
						</tr>
					</thead>
					<tfoot>
						<tr>
							<th><?php echo JText::_('PLG_SMARTSUB_OFFLINE_CONCEPT'); ?></th>
							<td><strong><span id="pr-offline-info-concept"></span></strong></td>
						</tr>
						<tr>
							<th><?php echo JText::_('PLG_SMARTSUB_OFFLINE_AMOUNT'); ?></th>
							<td><strong><span id="pr-offline-info-amount"></span></strong></td>
						</tr>
					</tfoot>
					<tbody>
					<?php foreach($options as $key => $value) : ?>
						<?php if(!empty($value)) : ?>
						<tr>
							<th><?php echo JText::_('PLG_SMARTSUB_OFFLINE_' . $key); ?></th>
							<td><?php echo htmlspecialchars($value); ?></td>
						<tr>
						<?php endif; ?>
					<?php endforeach; ?>
					</tbody>
				</table>	
			</div>
			<div class="modal-footer">
				<button type="button" class="close" data-dismiss="modal" aria-hidden="true"><?php echo JText::_('JCANCEL'); ?></button>
				<button type="button" onclick="pr_offline_account_data.submit(this);" class="close" data-dismiss="modal" aria-hidden="true"><?php echo JText::_('JSUBMIT'); ?></button>
			</div>
		</div>
<?php		
		$content = ob_get_clean();
		return $content;
	}
	
	private function getTransactionIDpattern()
	{
		$date_code = JDate::getInstance()->format('ymd');
		return $date_code . '-%s' . '%03d';
	}
}